package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.InfoDetailQueryParam;
import com.xdf.creative.base.vo.creative.InfoDetailQueryVo;
import com.xdf.creative.module.entity.InfoDetail;
import com.xdf.creative.module.service.InfoDetailService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 信息详情 前端控制器
 * </p>
 *
 * @author lgw
 * @since 2020-03-09
 */
@Slf4j
@RestController
@RequestMapping("/infoDetail")
@Api("信息详情 API")
public class InfoDetailController extends BaseController {

    @Autowired
    private InfoDetailService infoDetailService;

    /**
     * 添加信息详情
     */
    @PostMapping("/add")
    @ApiOperation(value = "添加InfoDetail对象", notes = "添加信息详情", response = ApiResult.class)
    public ApiResult<Boolean> addInfoDetail(@Valid @RequestBody InfoDetail infoDetail) throws Exception {
        boolean flag = infoDetailService.save(infoDetail);
        return ApiResult.result(flag);
    }

    /**
     * 修改信息详情
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改InfoDetail对象", notes = "修改信息详情", response = ApiResult.class)
    public ApiResult<Boolean> updateInfoDetail(@Valid @RequestBody InfoDetail infoDetail) throws Exception {
        boolean flag = infoDetailService.updateById(infoDetail);
        return ApiResult.result(flag);
    }

    /**
     * 删除信息详情
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除InfoDetail对象", notes = "删除信息详情", response = ApiResult.class)
    public ApiResult<Boolean> deleteInfoDetail(@Valid @RequestBody IdParam idParam) throws Exception {
        boolean flag = infoDetailService.removeById(idParam.getId());
        return ApiResult.result(flag);
    }

    /**
     * 获取信息详情
     */
    @PostMapping("/info")
    @ApiOperation(value = "获取InfoDetail对象详情", notes = "查看信息详情", response = InfoDetailQueryVo.class)
    public ApiResult<InfoDetailQueryVo> getInfoDetail(@Valid @RequestBody IdParam idParam) throws Exception {
        InfoDetailQueryVo infoDetailQueryVo = infoDetailService.getInfoDetailById(idParam.getId());
        return ApiResult.ok(infoDetailQueryVo);
    }

    /**
     * 信息详情分页列表
     */
    @PostMapping("/getPageList")
    @ApiOperation(value = "获取InfoDetail分页列表", notes = "信息详情分页列表", response = InfoDetailQueryVo.class)
    public ApiResult<PageTool<InfoDetailQueryVo>> getInfoDetailPageList(@Valid @RequestBody InfoDetailQueryParam infoDetailQueryParam) throws Exception {
        PageTool<InfoDetailQueryVo> pageList = infoDetailService.getInfoDetailPageList(infoDetailQueryParam);
        return ApiResult.ok(pageList);
    }

}

