package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.InfoRoleImpowerTypeQueryParam;
import com.xdf.creative.base.params.creative.InfoRoleTypeQueryParam;
import com.xdf.creative.base.vo.creative.InfoRoleTypeQueryVo;
import com.xdf.creative.module.entity.InfoRoleType;
import com.xdf.creative.module.service.InfoRoleTypeService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import com.xdf.creative.util.tree.TreeConfig;
import com.xdf.creative.util.tree.Utils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 角色类型 前端控制器
 * </p>
 *
 * @author lgw
 * @since 2020-04-02
 */
@Slf4j
@RestController
@RequestMapping("/infoRoleType")
@Api("角色类型 API")
public class InfoRoleTypeController extends BaseController {

    @Autowired
    private InfoRoleTypeService infoRoleTypeService;


    /**
     * 角色授权资源类型
     * @param infoRoleImpowerTypeQueryParam
     * @return
     * @throws Exception
     */
    @PostMapping("/setTypeToRole")
    @ApiOperation(value = "添加InfoRoleType对象", notes = "添加角色类型", response = ApiResult.class)
    public ApiResult setTypeToRole(@Valid @RequestBody InfoRoleImpowerTypeQueryParam infoRoleImpowerTypeQueryParam) throws Exception {
        return infoRoleTypeService.setTypeToRole(infoRoleImpowerTypeQueryParam);
    }



    /**
     * 角色授权查询资源树
     */
    @PostMapping("/impowerTypeTreeList")
    @ApiOperation(value = "获取授权菜单列表", notes = "获取所有菜单列表", response = Object.class)
    public ApiResult impowerTypeTreeList(@Valid @RequestBody IdParam idParam) throws Exception {
        List<Object> list = infoRoleTypeService.impowerTypeTreeList(idParam.getId());
        TreeConfig config =  new TreeConfig("typeId", "typeParentId", "typeParentId", "0", "");
        return ApiResult.ok(Utils.tree(list, config));
    }



    /**
     * 获取当前用户查询资源树
     */
    @PostMapping("/getCurrentTypeTreeList")
    @ApiOperation(value = "获取当前用户拥有菜单列表", notes = "获取所有菜单列表", response = Object.class)
    public ApiResult getCurrentTypeTreeList() throws Exception {
        List<Object> list = infoRoleTypeService.getRoleTypeTreeList();
        TreeConfig config =  new TreeConfig("typeId", "typeParentId", "typeParentId", "0", "");
        return ApiResult.ok(Utils.tree(list, config));
    }








}

