package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.InfoStoreAddParam;
import com.xdf.creative.base.params.creative.InfoStoreQueryParam;
import com.xdf.creative.base.params.creative.InfoStoreUrlParam;
import com.xdf.creative.base.vo.creative.InfoStoreQueryVo;
import com.xdf.creative.module.entity.InfoStore;
import com.xdf.creative.module.service.InfoStoreService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 文件库 前端控制器
 * </p>
 *
 * @author lgw
 * @since 2020-03-31
 */
@Slf4j
@RestController
@RequestMapping("/infoStore")
@Api("文件库 API")
public class InfoStoreController extends BaseController {

    @Autowired
    private InfoStoreService infoStoreService;


    @Value("${upload_basePath}")
    private String basePath;
    @Value("${wx_basePath}")
    private String wxPath;


    /**
     * 添加文件库
     */
    @PostMapping("/addStore")
    @ApiOperation(value = "添加InfoStore对象", notes = "添加文件库", response = ApiResult.class)
    public ApiResult addStore(@Valid @RequestBody InfoStoreAddParam infoStoreAddParam) throws Exception {
        return infoStoreService.addStore(infoStoreAddParam);
    }





    /**
     * 停用图片
     */
    @PostMapping("/delStore")
    @ApiOperation(value = "添加InfoStore对象", notes = "添加文件库", response = ApiResult.class)
    public ApiResult delStore(@Valid @RequestBody IdParam idParam) throws Exception {
        return infoStoreService.delStore(idParam.getId());
    }

    /**
     * 前台调用图片库
     */
    @PostMapping("/getUserStorePage")
    @ApiOperation(value = "获取InfoStore分页列表", notes = "文件库分页列表", response = InfoStoreQueryVo.class)
    public ApiResult<PageTool<InfoStoreQueryVo>> getUserStorePage(@Valid @RequestBody InfoStoreQueryParam infoStoreQueryParam) throws Exception {
        PageTool<InfoStoreQueryVo> pageList = infoStoreService.getUserStorePage(infoStoreQueryParam);
        return ApiResult.ok(pageList);
    }


    /**
     * 文件库分页列表
     */
    @PostMapping("/getPageList")
    @ApiOperation(value = "获取InfoStore分页列表", notes = "文件库分页列表", response = InfoStoreQueryVo.class)
    public ApiResult<PageTool<InfoStoreQueryVo>> getInfoStorePageList(@Valid @RequestBody InfoStoreQueryParam infoStoreQueryParam) throws Exception {
        PageTool<InfoStoreQueryVo> pageList = infoStoreService.getInfoStorePageList(infoStoreQueryParam);
        return ApiResult.ok(pageList);
    }


    /**
     * 前台调用图片库
     */
    @PostMapping("/getUrlStort")
    @ApiOperation(value = "获取InfoStore分页列表", notes = "文件库分页列表", response = InfoStoreQueryVo.class)
    public ApiResult getUrlStort(@Valid @RequestBody InfoStoreUrlParam infoStoreUrlParam) throws Exception {
        infoStoreUrlParam.setFilePath(basePath+wxPath);
        return infoStoreService.getUrlStort(infoStoreUrlParam);
    }












    /**
     * 添加文件库
     */
    @PostMapping("/add")
    @ApiOperation(value = "添加InfoStore对象", notes = "添加文件库", response = ApiResult.class)
    public ApiResult<Boolean> addInfoStore(@Valid @RequestBody InfoStore infoStore) throws Exception {
        boolean flag = infoStoreService.save(infoStore);
        return ApiResult.result(flag);
    }

    /**
     * 修改文件库
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改InfoStore对象", notes = "修改文件库", response = ApiResult.class)
    public ApiResult<Boolean> updateInfoStore(@Valid @RequestBody InfoStore infoStore) throws Exception {
        boolean flag = infoStoreService.updateById(infoStore);
        return ApiResult.result(flag);
    }

    /**
     * 删除文件库
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除InfoStore对象", notes = "删除文件库", response = ApiResult.class)
    public ApiResult<Boolean> deleteInfoStore(@Valid @RequestBody IdParam idParam) throws Exception {
        boolean flag = infoStoreService.removeById(idParam.getId());
        return ApiResult.result(flag);
    }

    /**
     * 获取文件库
     */
    @PostMapping("/info")
    @ApiOperation(value = "获取InfoStore对象详情", notes = "查看文件库", response = InfoStoreQueryVo.class)
    public ApiResult<InfoStoreQueryVo> getInfoStore(@Valid @RequestBody IdParam idParam) throws Exception {
        InfoStoreQueryVo infoStoreQueryVo = infoStoreService.getInfoStoreById(idParam.getId());
        return ApiResult.ok(infoStoreQueryVo);
    }


}

