package com.xdf.creative.module.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.SysOrganizeAddParam;
import com.xdf.creative.base.params.creative.SysOrganizeBackParam;
import com.xdf.creative.base.params.creative.SysOrganizeEditParam;
import com.xdf.creative.base.params.creative.SysOrganizeQueryParam;
import com.xdf.creative.base.vo.creative.SysOrganizeBackVo;
import com.xdf.creative.base.vo.creative.SysOrganizeQueryVo;
import com.xdf.creative.base.vo.creative.SysOrganizeTreeVo;
import com.xdf.creative.enums.OrganizeTypeEnum;
import com.xdf.creative.enums.StateEnum;
import com.xdf.creative.module.entity.SysOrganize;
import com.xdf.creative.module.service.SysOrganizeService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import com.xdf.creative.util.tree.TreeConfig;
import com.xdf.creative.util.tree.Utils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 组织信息 前端控制器
 * </p>
 *
 * @author DeanYe
 * @since 2019-11-05
 */
@Slf4j
@RestController
@RequestMapping("/sysOrganize")
@Api("组织信息 API")
public class SysOrganizeController extends BaseController {

    @Autowired
    private SysOrganizeService sysOrganizeService;


    /**
     * 添加组织信息
     */
    @PostMapping("/add")
    @ApiOperation(value = "添加SysOrganize对象", notes = "添加组织信息", response = ApiResult.class)
    public ApiResult addSysOrganize(@Valid @RequestBody SysOrganizeAddParam sysOrganizeAddParam) throws Exception {
        return sysOrganizeService.add(sysOrganizeAddParam);
    }

    /**
     * 启用停用组织
     */
    @PostMapping("/enabledOrganize")
    @ApiOperation(value = "添加SysOrganize对象", notes = "编辑文创办组织信息", response = ApiResult.class)
    public ApiResult enabledOrganize(@Valid @RequestBody IdParam idParam) throws Exception {
        return sysOrganizeService.enabledOrganize(idParam);
    }


    /**
     * 编辑文创办组织信息
     */
    @PostMapping("/editMange")
    @ApiOperation(value = "添加SysOrganize对象", notes = "编辑文创办组织信息", response = ApiResult.class)
    public ApiResult editMange(@Valid @RequestBody SysOrganizeEditParam sysOrganizeAddParam) throws Exception {
        return sysOrganizeService.editMange(sysOrganizeAddParam);
    }

    /**
     * 组织信息分页列表
     */
    @PostMapping("/getPageList")
    @ApiOperation(value = "获取SysOrganize分页列表", notes = "组织信息分页列表", response = SysOrganizeQueryVo.class)
    public ApiResult<PageTool<SysOrganizeQueryVo>> getSysOrganizePageList(@RequestBody SysOrganizeQueryParam sysOrganizeQueryParam) throws Exception {
        PageTool<SysOrganizeQueryVo> pageList = sysOrganizeService.getSysOrganizePageList(sysOrganizeQueryParam);
        return ApiResult.ok(pageList);
    }


    /**
     * 后台获取文创办组织
     */
    @PostMapping("/getManagePageList")
    @ApiOperation(value = "获取SysOrganize分页列表", notes = "组织信息分页列表", response = SysOrganizeQueryVo.class)
    public ApiResult<PageTool<SysOrganizeBackVo>> getManagePageList(@RequestBody SysOrganizeBackParam sysOrganizeBackParam) throws Exception {
        PageTool<SysOrganizeBackVo> pageList = sysOrganizeService.getManagePageList(sysOrganizeBackParam);
        return ApiResult.ok(pageList);
    }


    /**
     * 后台获取园区组织
     */
    @PostMapping("/getParkPageList")
    @ApiOperation(value = "获取SysOrganize分页列表", notes = "组织信息分页列表", response = SysOrganizeQueryVo.class)
    public ApiResult<PageTool<SysOrganizeBackVo>> getParkPageList(@RequestBody SysOrganizeBackParam sysOrganizeBackParam) throws Exception {
        PageTool<SysOrganizeBackVo> pageList = sysOrganizeService.getParkPageList(sysOrganizeBackParam);
        return ApiResult.ok(pageList);
    }


    /**
     * 组织机构Tree列表
     */
    @PostMapping("/getOrganizeTreeList")
    @ApiOperation(value = "获取栏目类型Tree列表", notes = "栏目类型Tree列表", response = SysOrganizeTreeVo.class)
    public ApiResult getOrganizeTreeList() throws Exception {
        List<Object> list = sysOrganizeService.getOrganizeTreeList();
        TreeConfig config = new TreeConfig("organizeId", "parOrganizeId", "parOrganizeId", sysOrganizeService.getRootOrganizeId(), "");
        return ApiResult.ok(Utils.tree(list, config));
    }


    /**
     * 添加园区用户 所需要的组织list
     *
     * @return
     * @throws Exception
     */
    @PostMapping("/getParkSelectList")
    @ApiOperation(value = "获取栏目类型Tree列表", notes = "栏目类型Tree列表", response = SysOrganizeTreeVo.class)
    public ApiResult getParkSelectList() throws Exception {
        List<SysOrganize> organizesList = sysOrganizeService.list(new QueryWrapper<SysOrganize>().eq("organize_type", OrganizeTypeEnum.ORGANIZE_PARK.getCode()).eq("state", StateEnum.ENABLE.getKey()));
        return ApiResult.ok(organizesList);
    }


    /**
     * 修改组织信息
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改SysOrganize对象", notes = "修改组织信息", response = ApiResult.class)
    public ApiResult<Boolean> updateSysOrganize(@Valid @RequestBody SysOrganize sysOrganize) throws Exception {
        boolean flag = sysOrganizeService.updateById(sysOrganize);
        return ApiResult.result(flag);
    }

    /**
     * 删除组织信息
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除SysOrganize对象", notes = "删除组织信息", response = ApiResult.class)
    public ApiResult<Boolean> deleteSysOrganize(@Valid @RequestBody IdParam idParam) throws Exception {
        boolean flag = sysOrganizeService.removeById(idParam.getId());
        return ApiResult.result(flag);
    }

    /**
     * 获取组织信息
     */
    @PostMapping("/getSysOrganize")
    @ApiOperation(value = "获取SysOrganize对象详情", notes = "查看组织信息", response = SysOrganizeQueryVo.class)
    public ApiResult<SysOrganizeQueryVo> getSysOrganize(@Valid @RequestBody IdParam idParam) throws Exception {
        SysOrganizeQueryVo sysOrganizeQueryVo = sysOrganizeService.getSysOrganizeById(idParam.getId());
        return ApiResult.ok(sysOrganizeQueryVo);
    }


}

