
package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.*;
import com.xdf.creative.base.vo.creative.SysUserBackQueryVo;
import com.xdf.creative.base.vo.creative.SysUserQueryVo;
import com.xdf.creative.enums.StatusCode;
import com.xdf.creative.module.service.OauthTokenService;
import com.xdf.creative.module.service.SysUserService;
import com.xdf.creative.shiro.jwt.JwtUtil;
import com.xdf.creative.support.aop.OperateLogger;
import com.xdf.creative.support.auth.LoginParam;
import com.xdf.creative.support.auth.LoginRefreshParam;
import com.xdf.creative.util.StringUtil;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * <pre>
 * 系统用户 前端控制器
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-24
 */
@Slf4j
@RestController
@RequestMapping("/sysUser")
@Api("系统用户 API")
public class SysUserController extends BaseController {

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private OauthTokenService oauthTokenService;

   /* @Value("${cas.server-url-prefix}")
    private String CAS_URL;*/

    /**
     * 用户登录
     *
     * @param loginParam
     * @param response
     * @return
     * @throws Exception
     */
    @PostMapping("/login")
    @ApiOperation(value = "登陆", notes = "系统用户登陆", response = ApiResult.class)
    public ApiResult login(@Valid @RequestBody LoginParam loginParam, HttpServletRequest request, HttpServletResponse response) throws Exception {
        return sysUserService.login(loginParam, request);
    }


    @PostMapping("/refresh")
    @ApiOperation(value = "登陆", notes = "系统用户登陆", response = ApiResult.class)
    public ApiResult refresh(@Valid @RequestBody LoginRefreshParam loginRefreshParam, HttpServletResponse response) throws Exception {
        return sysUserService.refresh(loginRefreshParam);
    }

    @PostMapping("/initUser")
    @ApiOperation(value = "初始化用户", notes = "初始化用户", response = ApiResult.class)
    public ApiResult initUser() throws Exception {
        return sysUserService.initUser();
    }


    /**
     * 用户登出
     *
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/logout")
    @ApiOperation(value = "登陆", notes = "系统用户登陆", response = ApiResult.class)
    public ApiResult logout(HttpServletRequest request) throws Exception {
        String token = request.getHeader("Authorization");
        Long userId = JwtUtil.getUserId(token);
        oauthTokenService.removeTokenByUserId(userId);
        return ApiResult.ok("退出成功");
    }

    /**
     * 验证用户是否存在
     */
    @PostMapping("/verifySysUser")
    @ApiOperation(value = " 验证用户是否存在", notes = " 验证用户是否存在", response = ApiResult.class)
    public ApiResult verifySysUser(@RequestBody SysUserAddQueryParam sysUserAddQueryParam) throws Exception {
        if (StringUtil.isEmpty(sysUserAddQueryParam.getUsername())) {
            return ApiResult.fail("请输入用户名！");
        }
        boolean isExists = sysUserService.isExistsByUsername(sysUserAddQueryParam.getUsername());
        if (isExists) {
            return ApiResult.error(StatusCode.SYS_USER_NAME_EXIST);
        } else {
            return ApiResult.ok();
        }

    }


    /**
     * 添加系统用户
     */
    @PostMapping("/addSysUser")
    @ApiOperation(value = "添加SysUser对象", notes = "添加系统用户", response = ApiResult.class)
    @OperateLogger(name = "添加用户", attributeId = "id", attributeName = "username")
    public ApiResult addSysUser(@Valid @RequestBody SysUserAddQueryParam sysUserAddQueryParam) throws Exception {
        return sysUserService.addSysUser(sysUserAddQueryParam);
    }


    /**
     * 停用启用用户
     */
    @PostMapping("/enabledSysUser")
    @ApiOperation(value = "添加SysUser对象", notes = "添加系统用户", response = ApiResult.class)
    @OperateLogger(name = "停用/启用用户", attributeId = "id", attributeName = "value")
    public ApiResult enabledSysUser(@Valid @RequestBody IdParam idParam) throws Exception {
        return sysUserService.enabledSysUser(idParam);
    }

    /**
     * 删除系统用户
     */
    @RequestMapping(value = "/deleteSysUser", method = {RequestMethod.GET, RequestMethod.POST})
    @ApiOperation(value = "删除SysUser对象", notes = "删除系统用户", response = ApiResult.class)
    @OperateLogger(name = "删除用户", attributeId = "id", attributeName = "value")
    public ApiResult<Boolean> deleteSysUser(@Valid @RequestBody IdParam idParam) throws Exception {
        return sysUserService.deleteSysUser(idParam);
    }

    /**
     * 修改密码
     */
    @PostMapping("/updatePassword")
    @ApiOperation(value = "修改密码", notes = "修改密码", response = ApiResult.class)
    @OperateLogger(name = "修改密码", attributeId = "userId", attributeName = "username")
    public ApiResult<Boolean> updatePassword(@Valid @RequestBody UpdatePasswordParam updatePasswordParam) throws Exception {
        return sysUserService.updatePassword(updatePasswordParam);
    }

    /**
     * 修改头像
     */
    @PostMapping("/updateSysUserHead")
    @ApiOperation(value = "修改头像", notes = "修改头像", response = ApiResult.class)
    @OperateLogger(name = "修改头像", attributeId = "id", attributeName = "username")
    public ApiResult updateSysUserHead(@Valid @RequestBody SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception {
        return sysUserService.updateSysUserHead(sysUserUpdateQueryParam);
    }

    /**
     * 后台修改用户信息（包含角色）
     */
    @PostMapping("/updateUserBack")
    @ApiOperation(value = "后台修改用户信息（包含角色）", notes = "后台修改用户信息（包含角色）", response = ApiResult.class)
    @OperateLogger(name = "管理员修改用户信息", attributeId = "id", attributeName = "username")
    public ApiResult updateUserBack(@Valid @RequestBody SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception {
        return sysUserService.updateUserBack(sysUserUpdateQueryParam);
    }


    /**
     * 用户个人信息修改
     */
    /**
     * 后台修改用户信息（包含角色）
     */
    @PostMapping("/updateUserNom")
    @ApiOperation(value = "个人修改用户信息", notes = "（包含角色", response = ApiResult.class)
    @OperateLogger(name = "个人修改用户信息", attributeId = "id", attributeName = "username")
    public ApiResult updateUserNom(@Valid @RequestBody SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception {
        return sysUserService.updateUserNom(sysUserUpdateQueryParam);
    }


    /**
     * 重置密码
     */
    @PostMapping("/resetPassword")
    @ApiOperation(value = "重置密码", notes = "重置密码", response = ApiResult.class)
    @OperateLogger(name = "重置密码", attributeId = "userId", attributeName = "username")
    public ApiResult resetPassword(@RequestBody UpdatePasswordParam updatePasswordParam) throws Exception {
        return sysUserService.resetPassword(updatePasswordParam);
    }


    /**
     * 后台文创办用户获取列表
     */
    @PostMapping("/getSysUserBackList")
    @ApiOperation(value = "获取SysUser分页列表", notes = "系统用户分页列表", response = SysUserQueryVo.class)
    public ApiResult<PageTool<SysUserBackQueryVo>> getSysUserBackList(@Valid @RequestBody SysUserBackQueryParam sysUserQueryParam) throws Exception {
        PageTool<SysUserBackQueryVo> pageList = sysUserService.getSysUserBackList(sysUserQueryParam);
        return ApiResult.ok(pageList);
    }


    /**
     * 获取系统用户token
     */
    @GetMapping("/getSysUserById")
    @ApiOperation(value = "获取SysUser对象详情", notes = "查看系统用户", response = SysUserQueryVo.class)
    public ApiResult getSysUser(HttpServletRequest request) throws Exception {
        return sysUserService.getSysUserById(request);
    }


    /**
     * 系统用户分页列表
     */
    @PostMapping("/getUserPageList")
    @ApiOperation(value = "获取SysUser分页列表", notes = "系统用户分页列表", response = SysUserQueryVo.class)
    public ApiResult<PageTool<SysUserQueryVo>> getSysUserPageList(@Valid @RequestBody SysUserQueryParam sysUserQueryParam) throws Exception {
        PageTool<SysUserQueryVo> pageList = sysUserService.getSysUserPageList(sysUserQueryParam);
        return ApiResult.ok(pageList);
    }


}

