package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.dt.WebParkEnterApplyParam;
import com.xdf.creative.base.params.dt.WebParkEnterBackParam;
import com.xdf.creative.base.params.dt.WebParkEnterQueryParam;
import com.xdf.creative.base.vo.dt.WebParkEnterQueryVo;
import com.xdf.creative.module.entity.WebParkEnter;
import com.xdf.creative.module.service.WebParkEnterService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 园区入会申请 前端控制器
 * </p>
 *
 * @author lgw
 * @since 2020-04-09
 */
@Slf4j
@RestController
@RequestMapping("/webParkEnter")
@Api("园区入会申请 API")
public class WebParkEnterController extends BaseController {

    @Autowired
    private WebParkEnterService webParkEnterService;

    /**
     * 添加园区入会申请
     */
    @PostMapping("/apply")
    @ApiOperation(value = "添加WebParkEnter对象", notes = "添加园区入会申请", response = ApiResult.class)
    public ApiResult apply(@Valid @RequestBody WebParkEnterApplyParam webParkEnterApplyParam) throws Exception {
        return webParkEnterService.apply(webParkEnterApplyParam);
    }

    /**
     * 审核园区入会申请
     */
    @PostMapping("/audit")
    @ApiOperation(value = "添加WebParkEnter对象", notes = "添加园区入会申请", response = ApiResult.class)
    public ApiResult audit(@Valid @RequestBody WebParkEnterBackParam webParkEnterBackParam) throws Exception {
        return webParkEnterService.audit(webParkEnterBackParam);
    }



    /**
     * 修改园区入会申请
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改WebParkEnter对象", notes = "修改园区入会申请", response = ApiResult.class)
    public ApiResult<Boolean> updateWebParkEnter(@Valid @RequestBody WebParkEnter webParkEnter) throws Exception {
        boolean flag = webParkEnterService.updateById(webParkEnter);
        return ApiResult.result(flag);
    }

    /**
     * 删除园区入会申请
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除WebParkEnter对象", notes = "删除园区入会申请", response = ApiResult.class)
    public ApiResult<Boolean> deleteWebParkEnter(@Valid @RequestBody IdParam idParam) throws Exception {
        boolean flag = webParkEnterService.removeById(idParam.getId());
        return ApiResult.result(flag);
    }

    /**
     * 获取园区入会申请
     */
    @PostMapping("/info")
    @ApiOperation(value = "获取WebParkEnter对象详情", notes = "查看园区入会申请", response = WebParkEnterQueryVo.class)
    public ApiResult<WebParkEnterQueryVo> getWebParkEnter(@Valid @RequestBody IdParam idParam) throws Exception {
        WebParkEnterQueryVo webParkEnterQueryVo = webParkEnterService.getWebParkEnterById(idParam.getId());
        return ApiResult.ok(webParkEnterQueryVo);
    }

    /**
     * 园区入会申请分页列表
     */
    @PostMapping("/getPageList")
    @ApiOperation(value = "获取WebParkEnter分页列表", notes = "园区入会申请分页列表", response = WebParkEnterQueryVo.class)
    public ApiResult<PageTool<WebParkEnterQueryVo>> getWebParkEnterPageList(@Valid @RequestBody WebParkEnterQueryParam webParkEnterQueryParam) throws Exception {
        PageTool<WebParkEnterQueryVo> pageList = webParkEnterService.getWebParkEnterPageList(webParkEnterQueryParam);
        return ApiResult.ok(pageList);
    }

}

