package com.xdf.creative.module.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.IsEnableSysOrganizeParam;
import com.xdf.creative.base.params.creative.SysOrganizeBackParam;
import com.xdf.creative.base.params.creative.SysOrganizeQueryParam;
import com.xdf.creative.base.vo.creative.SysOrganizeBackVo;
import com.xdf.creative.base.vo.creative.SysOrganizeQueryVo;
import com.xdf.creative.module.entity.SysOrganize;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.io.Serializable;
import java.util.List;

/**
 * 组织信息 Mapper 接口
 */
@Repository
public interface SysOrganizeMapper extends BaseMapper<SysOrganize> {


    /**
     * 获取组织机构树
     * @return
     */
    List<Object> getTreeOrganizeList(String regionId);


    /**
     * 根据用户id
     *
     * @param userId
     * @return
     */
    SysOrganizeQueryVo getSysOrganizeByUserId(Long userId);

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     */
    SysOrganizeQueryVo getSysOrganizeById(Serializable id);


    /**
     * 后台获取文创办管理列表
     * @param page
     * @param sysOrganizeBackParam
     * @return
     */
    IPage<SysOrganizeBackVo> getManagePageList(@Param("page") Page page, @Param("param") SysOrganizeBackParam sysOrganizeBackParam);


    /**
     * 后台获取园区管理列表
     * @param page
     * @param sysOrganizeBackParam
     * @return
     */
    IPage<SysOrganizeBackVo> getParkPageList(@Param("page") Page page, @Param("param") SysOrganizeBackParam sysOrganizeBackParam);




    /**
     * 获取分页对象
     *
     * @param page
     * @param sysOrganizeQueryParam
     * @return
     */
    IPage<SysOrganizeQueryVo> getSysOrganizePageList(@Param("page") Page page, @Param("param") SysOrganizeQueryParam sysOrganizeQueryParam);


    /**
     * 检查当前用户组织是否可用
     *
     * @param isEnableSysOrganizeParam
     * @return
     * @throws Exception
     */
    Integer isEnableSysOrganize(IsEnableSysOrganizeParam isEnableSysOrganizeParam) throws Exception;

}
