
package com.xdf.creative.module.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.SysPermissionCurrentUserParam;
import com.xdf.creative.base.params.creative.SysPermissionQueryParam;
import com.xdf.creative.base.vo.creative.SysPermissionQueryVo;
import com.xdf.creative.base.vo.creative.SysPermissionUserVo;
import com.xdf.creative.module.entity.InfoType;
import com.xdf.creative.module.entity.SysPermission;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.io.Serializable;
import java.util.List;

/**
 * <pre>
 * 系统权限 Mapper 接口
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-24
 */
@Repository
public interface SysPermissionMapper extends BaseMapper<SysPermission> {


    /**
     * 获取用户角色授权资源 树
     */
    List<Object> impowerResourceTreeList(@Param("roleId") Long roleId);






    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     */
    SysPermissionQueryVo getSysPermissionById(Serializable id);

    /**
     * 获取分页对象
     *
     * @param page
     * @param sysPermissionQueryParam
     * @return
     */
    IPage<SysPermissionQueryVo> getSysPermissionPageList(@Param("page") Page page, @Param("param") SysPermissionQueryParam sysPermissionQueryParam);


    /**
     * 根据用户id获取该用户所有Url
     * 一个用户可以映射多个角色
     *
     * @param userId
     * @return
     */
    List<String> getPermissionUrlByUserId(@Param("userId") Long userId);


    /**
     *
     * 根据用户Id获取权限
     * @param userId
     * @return
     */
    List<SysPermissionUserVo> getPermissionByUserId(@Param("userId") Long userId);



    /**
     * 根据用户id获取菜单列表
     *
     * @param userId
     * @return
     */
    List<SysPermission> getMenuListByUserId(@Param("userId") Long userId);



    List<SysPermission> currentUserResourceTreeList(@Param("param") SysPermissionCurrentUserParam sysPermissionCurrentUserParam);

    /**
     * 根据用户ID获取菜单()
     *
     * @param userId
     * @return
     */
    List<Object> getMenusByUserId(@Param("userId") Long userId);


    /**
     * 获取不需要鉴权urls
     *
     * @return
     */
    List<String> getSkipAuthUrls();


}
