package com.xdf.creative.module.service;

import com.xdf.creative.base.params.creative.*;
import com.xdf.creative.base.service.BaseService;
import com.xdf.creative.base.vo.creative.*;
import com.xdf.creative.module.entity.InfoContent;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;

import java.io.Serializable;
import java.util.List;

/**
 * <p>
 * 信息内容 服务类
 * </p>
 *
 * @author lgw
 * @since 2020-03-09
 */
public interface InfoContentService extends BaseService<InfoContent> {

    /**
     * 添加内容信息
     * @param infoContentAddQueryParam
     * @return
     */
    ApiResult addInfoContent(InfoContentAddQueryParam infoContentAddQueryParam);

    /**
     * 修改内容信息
     * @param infoContentAddQueryParam
     * @return
     */
    ApiResult updateInfoContent(InfoContentAddQueryParam infoContentAddQueryParam);


    /**
     * 管理员修改内容信息
     * @param infoContentEditQueryParam
     * @return
     */
    ApiResult editInfoManage(InfoContentEditQueryParam infoContentEditQueryParam);


    /**
     * 初审核内容信息
     * @param infoContentAuditQueryParam
     * @return
     */
    ApiResult firstAuditInfo(InfoContentAuditQueryParam infoContentAuditQueryParam);

    /**
     * 终审内容信息
     * @param infoContentAuditQueryParam
     * @return
     */
    ApiResult finalAuditInfo(InfoContentAuditQueryParam infoContentAuditQueryParam);

    /***
     * 提交内容信息
     */
    ApiResult commitInfoContent(InfoContentAuditQueryParam infoContentAuditQueryParam);

    /**
     * 上下线
     * @param infoContentAuditQueryParam
     * @return
     */
    ApiResult onlineInfoContent(InfoContentAuditQueryParam infoContentAuditQueryParam);

    /**
     * 是否置顶
     */
    ApiResult isTopInfoContent(InfoContentAuditQueryParam infoContentAuditQueryParam);

    /**
     * 修改排序号
     */
    ApiResult updateInfoSort(InfoContentAuditQueryParam infoContentAuditQueryParam);


    /**
     * 信息是否置为回收站
     */
    ApiResult recycleInfoContent(InfoContentAuditQueryParam infoContentAuditQueryParam);


    /**
     * 管理员回收站信息删除
     */
    ApiResult deleteInfoManageContent(InfoContentAuditQueryParam infoContentAuditQueryParam);



    /**
     * 首页显示新闻
     * @param infoContentHomePageQueryParam
     * @return
     */
    ApiResult getHomePageInfo(InfoContentHomePageQueryParam infoContentHomePageQueryParam);

    ApiResult getHomePagesInfo(InfoContentHomePageListParam infoContentHomePageListParam);




    /**
     * 获取新闻搜索
     * @param infoContentWebSearchQueryParam
     * @return
     * @throws Exception
     */
    PageTool<InfoContentWebSearchQueryVo> getSearchWebInfo(InfoContentWebSearchQueryParam infoContentWebSearchQueryParam) throws Exception;

    PageTool<InfoContentWebSearchQueryVo> getSearchWebsInfo(InfoContentWebSearchListParam infoContentWebSearchListParam) throws Exception;


    /**
     * 园区推荐列表
     * @param infoContentWebSearchQueryParam
     * @return
     * @throws Exception
     */
    List<InfoContentWebSearchQueryVo> getSearchParkRecommendWebInfo(InfoContentWebSearchQueryParam infoContentWebSearchQueryParam) throws Exception;


    /**
     * 手机端搜索新闻
     * @param infoContentWebSearchQueryParam
     * @return
     * @throws Exception
     */
    PageTool<InfoContentWebSearchMbQueryVo> getSearchMbInfo(InfoContentWebSearchQueryParam infoContentWebSearchQueryParam) throws Exception;


    /**
     * 后台发布列表
     */
    PageTool<InfoContentBackSearchQueryVo> getInfoContentByAdd(InfoContentBackSearchQueryParam infoContentBackSearchQueryParam) throws Exception;

    /**
     * 后台初审列表
     */
    PageTool<InfoContentBackSearchQueryVo> getInfoContentByFirstAudit(InfoContentBackSearchQueryParam infoContentBackSearchQueryParam) throws Exception;


    /**
     * 后台终审列表
     */
    PageTool<InfoContentBackSearchQueryVo> getInfoContentByFinalAudit(InfoContentBackSearchQueryParam infoContentBackSearchQueryParam) throws Exception;


    /**
     * 后台回收站列表
     * @param infoContentBackSearchQueryParam
     * @return
     * @throws Exception
     */
    PageTool<InfoContentRecycleSearchQueryVo> getInfoContentByRecycle(InfoContentBackSearchQueryParam infoContentBackSearchQueryParam) throws Exception;


    /**
     * 获取内容单条
     * @param id
     * @return
     */
    ApiResult getInfoContent(Serializable id);


    /**
     * 预览信息
     * @param id
     * @return
     */
    ApiResult getWebInfoContent(Serializable id);


    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     */
    InfoContentQueryVo getInfoContentById(Serializable id) throws Exception;




    /**
     * 获取分页对象
     *
     * @param infoContentQueryParam
     * @return
     */
    PageTool<InfoContentQueryVo> getInfoContentPageList(InfoContentQueryParam infoContentQueryParam) throws Exception;



    ApiResult getInfoHtml(String infoId);


    ApiResult getHtmlByInfo();


}
