
package com.xdf.creative.module.service;


import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.*;
import com.xdf.creative.base.service.BaseService;
import com.xdf.creative.base.vo.creative.SysUserBackQueryVo;
import com.xdf.creative.base.vo.creative.SysUserInfoQueryVo;
import com.xdf.creative.base.vo.creative.SysUserQueryVo;
import com.xdf.creative.module.entity.SysUser;
import com.xdf.creative.support.auth.LoginParam;
import com.xdf.creative.support.auth.LoginRefreshParam;
import com.xdf.creative.support.auth.LoginSysUserTokenVo;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import sun.security.util.AuthResources_pt_BR;

import javax.servlet.http.HttpServletRequest;
import java.io.Serializable;
import java.util.Set;

/**
 * <pre>
 * 系统用户 服务类
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-24
 */
public interface SysUserService extends BaseService<SysUser> {

    /**
     * 用户登录
     *
     * @param loginParam
     * @return
     * @throws Exception
     */
    ApiResult login(LoginParam loginParam, HttpServletRequest request) throws Exception;




    /**
     * 用户单点登录
     * @param loginParam
     * @param request
     * @return
     * @throws Exception
     */
    ApiResult loginExt(LoginParam loginParam, HttpServletRequest request) throws Exception;



    /**
     * 统一认证绑定
     * @param loginParam
     * @param request
     * @return
     * @throws Exception
     */

    ApiResult bond(LoginParam loginParam, HttpServletRequest request) throws Exception;

    /**
     * 后台绑定
     * @param loginParam
     * @param request
     * @return
     * @throws Exception
     */
    ApiResult bondBack(LoginParam loginParam, HttpServletRequest request) throws Exception;

    /**
     * 统一认证获取用户信息
     * @param request
     * @return
     * @throws Exception
     */
    ApiResult getUserInfo(HttpServletRequest request) throws Exception;


    /**
     * 统一认证后台用户信息
     * @param request
     * @return
     * @throws Exception
     */
    ApiResult loginBackIndex(HttpServletRequest request) throws Exception;




    /**
     * 刷新token
     * @param loginRefreshParam
     * @return
     * @throws Exception
     */
    ApiResult refresh(LoginRefreshParam loginRefreshParam) throws Exception;

    /**
     * 添加文创办用户
     * @return
     * @throws Exception
     */
    ApiResult addSysUser(SysUserAddQueryParam sysUserAddQueryParam)throws Exception;


    /**
     * 停用启用用户
     * @param idParam
     * @return
     */
    ApiResult enabledSysUser(IdParam idParam);

    /**
     * 删除用户
     * @param idParam
     * @return
     */
    ApiResult deleteSysUser(IdParam idParam);

    /**
     * 修改密码
     *
     * @param updatePasswordParam
     * @return
     * @throws Exception
     */
    ApiResult updatePassword(UpdatePasswordParam updatePasswordParam) throws Exception;

    /**
     * 修改系统用户头像
     *
     * @param sysUserUpdateQueryParam
     * @return
     * @throws Exception
     */
    ApiResult updateSysUserHead(SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception;

    /**
     * 后台修改用户信息
     *
     * @param sysUserUpdateQueryParam
     * @return
     * @throws Exception
     */
    ApiResult updateUserBack(SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception;


    /**
     * 用户修改个人信息
     *
     * @param sysUserUpdateQueryParam
     * @return
     * @throws Exception
     */
    ApiResult updateUserNom(SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception;


    /**
     * 重置密码
     * @param updatePasswordParam
     * @return
     * @throws Exception
     */
    ApiResult resetPassword(UpdatePasswordParam updatePasswordParam) throws Exception;

    /**
     * 后台用户分页
     *
     * @param sysUserQueryParam
     * @return
     * @throws Exception
     */
    PageTool<SysUserBackQueryVo> getSysUserBackList(SysUserBackQueryParam sysUserQueryParam) throws Exception;



    /**
     * 根据用户id获取用户信息
     * @return
     */
    ApiResult getSysUserById(HttpServletRequest request);



    /**
     * 获取分页对象
     *
     * @param sysUserQueryParam
     * @return
     * @throws Exception
     */
    PageTool<SysUserQueryVo> getSysUserPageList(SysUserQueryParam sysUserQueryParam) throws Exception;

    /**
     * 判断用户名是否存在
     *
     * @param username
     * @return
     * @throws Exception
     */
    boolean isExistsByUsername(String username) throws Exception;

    /**
     * 检验部门和角色是否存在并且已启用
     *
     * @param departmentId
     * @param roleId
     * @throws Exception
     */
    void checkOrganizeAndRole(Long departmentId, Set<Long> roleId) throws Exception;

    /**
     * 通过角色id判断是否存在可用用户id
     *
     * @param roleId
     * @return
     * @throws Exception
     */
    boolean isExistsSysUserByRoleId(Long roleId) throws Exception;



    SysUser getSysUserByUserName(String username) throws Exception;



    ApiResult initUser();
}
