package com.xdf.creative.module.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.InfoWordManageQueryParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.InfoWordManageQueryVo;
import com.xdf.creative.enums.StatusCode;
import com.xdf.creative.module.entity.InfoWordManage;
import com.xdf.creative.module.mapper.InfoWordManageMapper;
import com.xdf.creative.module.service.InfoWordManageService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Date;


/**
 * <p>
 * 词管理 服务实现类
 * </p>
 *
 * @author lgw
 * @since 2020-03-06
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class InfoWordManageServiceImpl extends BaseServiceImpl<InfoWordManageMapper, InfoWordManage> implements InfoWordManageService {

    @Autowired
    private InfoWordManageMapper infoWordManageMapper;

    /**
     * 添加维护内容
     * @param infoWordManage
     * @return
     */
    @Override
    public ApiResult addInfoWordManage(InfoWordManage infoWordManage) {
        infoWordManage.setAccendantUser(getUserId());
        infoWordManageMapper.insert(infoWordManage);
        return ApiResult.ok();
    }

    /**
     * 修改维护内容
     * @param infoWordManage
     * @return
     */
    @Override
    public ApiResult updateInfoWordManage(InfoWordManage infoWordManage) {
        if(null==infoWordManage.getId()){
            return ApiResult.error(StatusCode.FORM_SUBMIT_ID_VERIFY);
        }
        InfoWordManage infoWordManageNew = this.getById(infoWordManage.getId());
        if (!verifyUserPermission(infoWordManageNew.getAccendantUser())) {
            return ApiResult.error(StatusCode.AUTH_UPDATE_VERIFY_ERROR);
        }
        BeanUtils.copyProperties(infoWordManage, infoWordManageNew);
        infoWordManageNew.setUpdateTime(new Date());
        this.saveOrUpdate(infoWordManageNew);
        return ApiResult.ok();
    }

    /**
     * 删除
     * @param idParam
     * @return
     */
    @Override
    public ApiResult deleteInfoWordManage(IdParam idParam) {
        if(null==idParam.getId()){
            return ApiResult.error(StatusCode.FORM_SUBMIT_ID_VERIFY);
        }
        InfoWordManage infoWordManageNew =this.getById(idParam.getId());
        if (!verifyUserPermission(infoWordManageNew.getAccendantUser())) {
            return ApiResult.error(StatusCode.AUTH_UPDATE_VERIFY_ERROR);
        }
        if(this.removeById(idParam.getId())){
            return ApiResult.ok();
        }else{
            return ApiResult.error(StatusCode.FORM_SUBMIT_DELETE_VERIFY);
        }
    }

    @Override
    public InfoWordManageQueryVo getInfoWordManageById(Serializable id) throws Exception {
        return infoWordManageMapper.getInfoWordManageById(id);
    }

    @Override
    public PageTool<InfoWordManageQueryVo> getInfoWordManagePageList(InfoWordManageQueryParam infoWordManageQueryParam) throws Exception {
        Page page = setPageParam(infoWordManageQueryParam);
        IPage<InfoWordManageQueryVo> iPage = infoWordManageMapper.getInfoWordManagePageList(page, infoWordManageQueryParam);
        return new PageTool(iPage);
    }

}
