package com.xdf.creative.module.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.SysRegionQueryParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.SysRegionQueryVo;
import com.xdf.creative.constant.CommonConstant;
import com.xdf.creative.enums.OrganizeTypeEnum;
import com.xdf.creative.module.entity.SysRegion;
import com.xdf.creative.module.mapper.SysRegionMapper;
import com.xdf.creative.module.service.SysRegionService;
import com.xdf.creative.util.page.PageTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.io.Serializable;
import java.util.List;
/**
 * <p>
 * 全国行政区域[sys_region] 服务实现类
 * </p>
 *
 * @author DeanYe
 * @since 2019-11-06
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class SysRegionServiceImpl extends BaseServiceImpl<SysRegionMapper, SysRegion> implements SysRegionService {

    @Autowired
    private SysRegionMapper sysRegionMapper;

    /**
     * 获取行政区域下拉树
     * @return
     * @throws Exception
     */
    @Override
    public List<Object> getRegionTreeList() throws Exception {
        return sysRegionMapper.getTreeRegionList();
    }


    /**
     * 获取行政区域下拉树(带授权)
     * @return
     * @throws Exception
     */
    @Override
    public List<Object> getRoleTreeList() throws Exception {
        String regionId = getRegionId(null);
        return sysRegionMapper.getRoleTreeList(regionId);
    }

    /**
     * 获取树的根节点
     * @return
     * @throws Exception
     */
    @Override
    public String getRootRegionId() throws Exception {
        if(OrganizeTypeEnum.ORGANIZE_DISTRICT.getCode().equals(getOrganizeType())){
            return CommonConstant.REGION_ROOT_CODE;
        }else{
            return "0";
        }
    }


    @Override
    public SysRegionQueryVo getSysRegionById(Serializable id) throws Exception {
        return sysRegionMapper.getSysRegionById(id);
    }

    /**
     * 不分页查询
     *
     * @param sysRegionQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public List<SysRegionQueryVo> getSysRegionList(SysRegionQueryParam sysRegionQueryParam) {
        return sysRegionMapper.getSysRegionList(sysRegionQueryParam);
    }

    @Override
    public PageTool<SysRegionQueryVo> getSysRegionPageList(SysRegionQueryParam sysRegionQueryParam) throws Exception {
        Page page = setPageParam(sysRegionQueryParam);
        IPage<SysRegionQueryVo> iPage = sysRegionMapper.getSysRegionPageList(page, sysRegionQueryParam);
        return new PageTool(iPage);
    }

}
