
package com.xdf.creative.module.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.*;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.*;
import com.xdf.creative.base.vo.tyrz.SysUserCurrentInfoVo;
import com.xdf.creative.constant.CommonConstant;
import com.xdf.creative.enums.*;
import com.xdf.creative.module.entity.*;
import com.xdf.creative.module.mapper.SysRegionMapper;
import com.xdf.creative.module.mapper.SysUserMapper;
import com.xdf.creative.module.service.*;
import com.xdf.creative.shiro.jwt.JwtProperties;
import com.xdf.creative.shiro.jwt.JwtUtil;
import com.xdf.creative.shiro.jwt.SaltUtil;
import com.xdf.creative.support.auth.LoginParam;
import com.xdf.creative.support.auth.LoginRefreshParam;
import com.xdf.creative.support.auth.LoginSysUserTokenVo;
import com.xdf.creative.support.auth.LoginSysUserVo;
import com.xdf.creative.support.exception.BusinessException;
import com.xdf.creative.support.exception.SysLoginException;
import com.xdf.creative.util.*;
import com.xdf.creative.util.convert.SysUserConvert;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.jasig.cas.client.util.AbstractCasFilter;
import org.jasig.cas.client.validation.Assertion;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.time.Duration;
import java.util.*;


/**
 * <pre>
 * 系统用户 服务实现类
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-24
 */
@Api
@Slf4j
@Service
public class SysUserServiceImpl extends BaseServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    private SysUserMapper sysUserMapper;

    @Lazy
    @Autowired
    private SysRoleService sysRoleService;

    @Lazy
    @Autowired
    private SysOrganizeService sysOrganizeService;

    @Lazy
    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private JwtProperties jwtProperties;

    @Autowired
    private OauthTokenService oauthTokenService;

    @Autowired
    private SysPermissionService sysPermissionService;

    @Autowired
    private SysAccountLogService sysAccountLogService;

    @Autowired
    private SysRegionService sysRegionService;

    @Autowired
    private SysRegionMapper sysRegionMapper;


    @Autowired
    private RedisTemplate redisTemplate;

   // @Value("${tyrz.app_id}")
    private String appId;

   // @Value("${tyrz.app_key}")
    private String appKey;

  //  @Value("${tyrz_user_info_url}")
    private String infoUrl;

  //  @Value("${tyrz_user_back_info_url}")
    private String infoBackUrl;

   // @Value("${tyrz_user_bond_url}")
    private String bondUrl;

   // @Value("${tyrz_user_bond_back_url}")
    private String bondBackUrl;

   // @Value("${cas.server-url-prefix}")
    private String CAS_URL;


    /*登录*/
    @Override
    @Transactional
    public ApiResult login(LoginParam loginParam, HttpServletRequest request) throws Exception {
        // 校验验证码

        if (StringUtil.isEmpty(loginParam.getVerifyCodeActual() )){
            return ApiResult.fail("验证码不能为空！");
        }

        String encrypt = PasswordUtil.encrypt(loginParam.getUsername()+loginParam.getType(), CommonConstant.VERIFY_CODE_KEY);
            String key = String.format(CommonConstant.VERIFY_CODE_GET, loginParam.getUsername(), encrypt);
            String cacheVerifyCode = (String) redisTemplate.opsForValue().get(key);

            if (!loginParam.getVerifyCodeActual().equalsIgnoreCase(cacheVerifyCode)) {
                return ApiResult.fail("验证码错误！");
            }



        String userName = loginParam.getUsername();
        SysUser sysUser = getSysUserByUserName(userName);
        if (sysUser == null) {
            log.error("登陆失败，用户名不正确,loginParam:{}", loginParam);
            this.loginFail(userName, loginParam.getPassword(), "登陆失败，用户名不正确!");
            return ApiResult.fail(ApiCode.USERNAME_ERROR);
        }
        if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
            log.debug("登陆失败，用户不可用,loginParam:{}", loginParam);
            this.loginFail(userName, loginParam.getPassword(), "登陆失败，用户不可用!");
            return ApiResult.fail(ApiCode.USERNAME_ERROR);
        }

        String encryptPassword = PasswordUtil.encrypt(loginParam.getPassword(), sysUser.getSalt());
        if (!encryptPassword.equals(sysUser.getPassword())) {
            this.loginFail(userName, loginParam.getPassword(), ApiCode.PASSWORD_ERROR.getMsg());
            throw new SysLoginException(ApiCode.USERNAME_ERROR);
        }

        SysOrganizeQueryVo sysOrganizeQueryVo = sysOrganizeService.getSysOrganizeById(sysUser.getOrganizeId());
        if (sysOrganizeQueryVo == null) {
            return ApiResult.fail("组织不存在");
        }
        if (!StateEnum.ENABLE.getKey().equals(sysOrganizeQueryVo.getState())) {
            return ApiResult.fail("组织已禁用");
        }

        SysRegion sysRegion = sysRegionService.getById(sysOrganizeQueryVo.getPopedomId());
        if (sysRegion == null) {
            return ApiResult.fail("所属辖区不存在");
        }
        // 将系统用户对象转换成登陆用户对象
        LoginSysUserVo loginSysUserVo = SysUserConvert.INSTANCE.sysUserToLoginSysUserVo(sysUser);
        loginSysUserVo.setOrganizeId(sysOrganizeQueryVo.getId());

        //获取角色id、名称、code
        HashSet<Long> roleIdSet = new HashSet<>();
        HashSet<String> roleNameSet = new HashSet<>();
        HashSet<String> roleCodeSet = new HashSet<>();

        List<SysRoleQueryVo> sysRoleQueryVoList = sysRoleService.getUserRoleByUserId(sysUser.getId());
        if (CollectionUtil.isEmpty(sysRoleQueryVoList)) {
            this.loginFail(userName, loginParam.getPassword(), "角色不存在!");
            return ApiResult.fail("角色不存在");
        }
        sysRoleQueryVoList.stream().forEach(sysRoleQueryVo -> {
            roleIdSet.add(sysRoleQueryVo.getId());
            //   roleNameSet.add(sysRoleQueryVo.getName());
            roleCodeSet.add(sysRoleQueryVo.getCode());
        });
        loginSysUserVo.setRoleId(roleIdSet).setRoleCode(roleCodeSet);
        //  .setRoleName(roleNameSet)
        //获取角色url
        //         Set<String> permissionCodesApiResult = sysPermissionService.getPermissionUrlByUserId(sysUser.getId());

        Map<String, Object> map = sysPermissionService.getPermissionByUserId(sysUser.getId());
        Set<String> permissionCodesApiResult = (Set<String>) map.get("url");
        List<String> resourceList = (List<String>) map.get("resource");

        loginSysUserVo.setResourceList(resourceList);
//        loginSysUserVo.setPermissionUrls(permissionCodesApiResult);

        // 获取数据库中保存的盐值
        String newSalt = SaltUtil.getSalt(sysUser.getSalt(), jwtProperties);
        // 生成token字符串并返回
        Long expireSecond = jwtProperties.getExpireSecond();
        String token = JwtUtil.generateToken(sysUser.getId(), userName, sysUser.getOrganizeId(), sysUser.getUserType(), sysRegion.getRegionId(), newSalt, Duration.ofSeconds(expireSecond));
        log.debug("token:{}", token);
        // 创建AuthenticationToken
        OauthToken oauthToken = new OauthToken();
        oauthTokenService.removeTokenByUserId(sysUser.getId());
        oauthToken.setAccessToken(token);
        if (!sysRoleQueryVoList.isEmpty()) {
            oauthToken.setRoles(loginSysUserVo.getRoleCode().toString());
        }
        oauthToken.setRefreshToken(StateEnum.DISABLE.getKey());
        oauthToken.setUserId(sysUser.getId());
        if (!permissionCodesApiResult.isEmpty()) {
            oauthToken.setPermissions(permissionCodesApiResult.toString());
        }

        if (permissionCodesApiResult.stream().filter(s -> s.contains("showData")).count() > 0) {
            loginSysUserVo.setDateAuth(true);
        } else {
            loginSysUserVo.setDateAuth(false);
        }

        oauthToken.setExpireTime(JwtUtil.getExpireDate(token));
        oauthTokenService.save(oauthToken);

        // 返回token和登陆用户信息对象
        LoginSysUserTokenVo loginSysUserTokenVo = new LoginSysUserTokenVo();
        loginSysUserTokenVo.setToken(token);
        loginSysUserTokenVo.setLoginSysUserVo(loginSysUserVo);
        this.loginSuccess(sysUser.getId());
        return ApiResult.ok(loginSysUserTokenVo);
    }

    @Override
    @Transactional
    public ApiResult loginExt(LoginParam loginParam, HttpServletRequest request) throws Exception {
        SysUser sysUser = this.getById(loginParam.getUserId());
        if (sysUser == null) {
            log.error("登陆失败，用户名不正确,loginParam:{}", loginParam);
            this.loginFail(sysUser.getUsername(), loginParam.getPassword(), "登陆失败，用户名不正确!");
            return ApiResult.fail(ApiCode.USERNAME_ERROR);
        }
        if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
            log.debug("登陆失败，用户不可用,loginParam:{}", loginParam);
            this.loginFail(sysUser.getUsername(), loginParam.getPassword(), "登陆失败，用户不可用!");
            return ApiResult.fail(ApiCode.USER_DISABLE);
        }
        SysOrganizeQueryVo sysOrganizeQueryVo = sysOrganizeService.getSysOrganizeById(sysUser.getOrganizeId());
        if (sysOrganizeQueryVo == null) {
            return ApiResult.fail("组织不存在");
        }
        if (!StateEnum.ENABLE.getKey().equals(sysOrganizeQueryVo.getState())) {
            return ApiResult.fail("组织已禁用");
        }
        SysRegion sysRegion = sysRegionService.getById(sysOrganizeQueryVo.getPopedomId());
        if (sysRegion == null) {
            return ApiResult.fail("所属辖区不存在");
        }

        // 将系统用户对象转换成登陆用户对象
        LoginSysUserVo loginSysUserVo = SysUserConvert.INSTANCE.sysUserToLoginSysUserVo(sysUser);
        loginSysUserVo.setOrganizeId(sysOrganizeQueryVo.getId());

        //获取角色id、名称、code
        HashSet<Long> roleIdSet = new HashSet<>();
        HashSet<String> roleNameSet = new HashSet<>();
        HashSet<String> roleCodeSet = new HashSet<>();

        List<SysRoleQueryVo> sysRoleQueryVoList = sysRoleService.getUserRoleByUserId(sysUser.getId());
        if (CollectionUtil.isEmpty(sysRoleQueryVoList)) {
            this.loginFail(sysUser.getUsername(), loginParam.getPassword(), "角色不存在!");
            return ApiResult.fail("角色不存在");
        }
        sysRoleQueryVoList.stream().forEach(sysRoleQueryVo -> {
            roleIdSet.add(sysRoleQueryVo.getId());
            //   roleNameSet.add(sysRoleQueryVo.getName());
            roleCodeSet.add(sysRoleQueryVo.getCode());
        });
        loginSysUserVo.setRoleId(roleIdSet)
                //  .setRoleName(roleNameSet)
                .setRoleCode(roleCodeSet);


        //获取角色url
        // Set<String> permissionCodesApiResult = sysPermissionService.getPermissionUrlByUserId(sysUser.getId());

        Map<String, Object> map = sysPermissionService.getPermissionByUserId(sysUser.getId());
        Set<String> permissionCodesApiResult = (Set<String>) map.get("url");
        List<String> resourceList = (List<String>) map.get("resource");

        loginSysUserVo.setResourceList(resourceList);
//        loginSysUserVo.setPermissionUrls(permissionCodesApiResult);

        // 获取数据库中保存的盐值
        String newSalt = SaltUtil.getSalt(sysUser.getSalt(), jwtProperties);
        // 生成token字符串并返回
        Long expireSecond = jwtProperties.getExpireSecond();
        String token = JwtUtil.generateToken(sysUser.getId(), sysUser.getUsername(), sysUser.getOrganizeId(), sysUser.getUserType(), sysRegion.getRegionId(), newSalt, Duration.ofSeconds(expireSecond));
        log.debug("token:{}", token);
        // 创建AuthenticationToken
        OauthToken oauthToken = new OauthToken();
        oauthTokenService.removeTokenByUserId(sysUser.getId());
        oauthToken.setAccessToken(token);
        if (!sysRoleQueryVoList.isEmpty()) {
            oauthToken.setRoles(loginSysUserVo.getRoleCode().toString());
        }
        oauthToken.setRefreshToken(StateEnum.DISABLE.getKey());
        oauthToken.setUserId(sysUser.getId());
        if (!permissionCodesApiResult.isEmpty()) {
            oauthToken.setPermissions(permissionCodesApiResult.toString());
        }

        if (permissionCodesApiResult.stream().filter(s -> s.contains("showData")).count() > 0) {
            loginSysUserVo.setDateAuth(true);
        } else {
            loginSysUserVo.setDateAuth(false);
        }

        oauthToken.setExpireTime(JwtUtil.getExpireDate(token));
        oauthTokenService.save(oauthToken);

        // 返回token和登陆用户信息对象
        LoginSysUserTokenVo loginSysUserTokenVo = new LoginSysUserTokenVo();
        loginSysUserTokenVo.setToken(token);
        loginSysUserTokenVo.setLoginSysUserVo(loginSysUserVo);
        this.loginSuccess(sysUser.getId());
        return ApiResult.ok(loginSysUserTokenVo);
    }


    /**
     * tydl认证
     *
     * @param request
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public ApiResult loginBackIndex(HttpServletRequest request) throws Exception {
        HttpSession session = request.getSession(false);
        if (null != session) {
            Assertion assertion = (Assertion) session.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION);
            if (assertion != null) {
                String username = assertion.getPrincipal().getName();
                Long userId = Long.parseLong((String) assertion.getPrincipal().getAttributes().get("id"));
                if (null == userId) {
                    ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
                }
                Map<String, Object> map = new HashMap<>();
                map.put("userId", userId);
                map.put("appId", appId);
                map.put("appKey", DigestUtils.sha256Hex(appKey));
                log.error("登录获取session" + CookieUtils.getCookie(request, "JSESSIONID"));
                log.error("登录获取session1" + CookieUtils.getCookie(request, "JSESSIONID_01"));
                if (!StringUtil.isEmpty(CookieUtils.getCookie(request, "JSESSIONID"))) {
                    map.put("sessionId", CookieUtils.getCookie(request, "JSESSIONID"));
                } else {
                    map.put("sessionId", CookieUtils.getCookie(request, "JSESSIONID_01"));
                }
                ApiResult result = HttpClient.sendGetData(infoBackUrl, "utf-8", map);
                if (ApiCode.SUCCESS.getCode() == result.getCode()) {
                    /* String userInfo = SymmetricEncoder.AESDncode(appId, result.getData().toString());*/
                    String userInfo = result.getData().toString();
                    SysUserCurrentInfoVo currentInfoVo = JSONObject.parseObject(userInfo, SysUserCurrentInfoVo.class);
                    if (null == currentInfoVo.getExtInfo()) {
                        return ApiResult.error(StatusCode.TYRZ_LOGIN_USER_BOND_VERIFY_FAILE).setData(username);
                    }
                    Long loginUserId = Long.parseLong(currentInfoVo.getExtInfo().getAppUserId());
                    LoginParam loginParam = new LoginParam();
                    loginParam.setUserId(loginUserId);
                    return this.loginExt(loginParam, request);
                } else {
                    return result;
                }
            } else {
                return ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
            }
        } else {
            return ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
        }
    }

    /**
     * 用户绑定
     *
     * @param loginParam
     * @param request
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public ApiResult bond(LoginParam loginParam, HttpServletRequest request) throws Exception {
        HttpSession session = request.getSession(false);
        Assertion assertion = (Assertion) session.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION);
        if (assertion != null) {
            Long userId = Long.parseLong((String) assertion.getPrincipal().getAttributes().get("id"));
            if (null == userId) {
                ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
            }
            String userName = loginParam.getUsername();
            SysUser sysUser = getSysUserByUserName(userName);
            if (sysUser == null) {
                log.error("登陆失败，用户名不正确,loginParam:{}", loginParam);
                return ApiResult.fail(ApiCode.USERNAME_ERROR);
            }
            if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
                log.debug("登陆失败，用户不可用,loginParam:{}", loginParam);
                return ApiResult.fail(ApiCode.USERNAME_ERROR);
            }

            String encryptPassword = PasswordUtil.encrypt(loginParam.getPassword(), sysUser.getSalt());
            if (!encryptPassword.equals(sysUser.getPassword())) {
                this.loginFail(userName, loginParam.getPassword(), ApiCode.PASSWORD_ERROR.getMsg());
                throw new SysLoginException(ApiCode.USERNAME_ERROR);
            }
            Map<String, Object> bondMap = new HashMap<>();
            bondMap.put("userId", userId);
            bondMap.put("appId", appId);
            bondMap.put("appKey", DigestUtils.sha256Hex(appKey));
            bondMap.put("appUserId", sysUser.getId());
            bondMap.put("sessionId", CookieUtils.getCookie(request, "JSESSIONID"));
            ApiResult result = HttpClient.sendPostByMap(bondUrl, bondMap, "utf-8");
            return result;
        } else {
            return ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
        }
    }

    /**
     * 后台用户绑定
     *
     * @param loginParam
     * @param request
     * @return
     * @throws Exception
     */
    @Override
    public ApiResult bondBack(LoginParam loginParam, HttpServletRequest request) throws Exception {
        HttpSession session = request.getSession(false);
        Assertion assertion = (Assertion) session.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION);
        if (assertion != null) {
            Long userId = Long.parseLong((String) assertion.getPrincipal().getAttributes().get("id"));
            if (null == userId) {
                ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
            }
            String userName = loginParam.getUsername();
            SysUser sysUser = getSysUserByUserName(userName);
            if (sysUser == null) {
                log.error("登陆失败，用户名不正确,loginParam:{}", loginParam);
                this.loginFail(userName, loginParam.getPassword(), "登陆失败，用户名不正确(认证绑定)!");
                return ApiResult.fail(ApiCode.USERNAME_ERROR);
            }
            if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
                log.debug("登陆失败，用户不可用,loginParam:{}", loginParam);
                this.loginFail(userName, loginParam.getPassword(), "登陆失败，用户不可用（认证绑定）!");
                return ApiResult.fail(ApiCode.USERNAME_ERROR);
            }

            String encryptPassword = PasswordUtil.encrypt(loginParam.getPassword(), sysUser.getSalt());
            if (!encryptPassword.equals(sysUser.getPassword())) {
                this.loginFail(userName, loginParam.getPassword(), ApiCode.PASSWORD_ERROR.getMsg() + "(认证绑定)");
                throw new SysLoginException(ApiCode.USERNAME_ERROR);
            }
            Map<String, Object> bondMap = new HashMap<>();
            bondMap.put("userId", userId);
            bondMap.put("appId", appId);
            bondMap.put("appKey", DigestUtils.sha256Hex(appKey));
            bondMap.put("appUserId", sysUser.getId());
            bondMap.put("sessionId", CookieUtils.getCookie(request, "JSESSIONID"));
            ApiResult result = HttpClient.sendPostByMap(bondBackUrl, bondMap, "utf-8");
            return result;
        } else {
            return ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
        }
    }


    /**
     * 统一登录认证获取基本信息
     *
     * @param request
     * @return
     * @throws Exception
     */
    @Override
    public ApiResult getUserInfo(HttpServletRequest request) throws Exception {
        HttpSession session = request.getSession(false);
        if (null != session) {
            Assertion assertion = (Assertion) session.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION);
            if (assertion != null) {
                String username = assertion.getPrincipal().getName();
                Long userId = Long.parseLong((String) assertion.getPrincipal().getAttributes().get("id"));
                if (null == userId) {
                    ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
                }
                Map<String, Object> map = new HashMap<>();
                map.put("userId", userId);
                map.put("appId", appId);
                map.put("appKey", DigestUtils.sha256Hex(appKey));
                map.put("sessionId", CookieUtils.getCookie(request, "JSESSIONID")); //request.getParameter("jsessionid"));
                ApiResult result = HttpClient.sendGetData(infoUrl, "utf-8", map);
                if (ApiCode.SUCCESS.getCode() == result.getCode()) {
                    /* String userInfo = SymmetricEncoder.AESDncode(appId, result.getData().toString());*/
                    String userInfo = result.getData().toString();
                    SysUserCurrentInfoVo currentInfoVo = JSONObject.parseObject(userInfo, SysUserCurrentInfoVo.class);
                    return ApiResult.ok(currentInfoVo);
                } else {
                    return result;
                }
            } else {
                return ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
            }
        } else {
            return ApiResult.error(StatusCode.TYRZ_LOGIN_OUT_FAILE);
        }
    }


    @Override
    @Transactional
    public ApiResult refresh(LoginRefreshParam loginRefreshParam) throws Exception {
        SysUser sysUser = this.getById(loginRefreshParam.getUserId());
        if (null == sysUser || StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
            return ApiResult.fail("用户不存在或已被停用！");
        }
        OauthTokenQueryVo oauthTokenOld = oauthTokenService.getOauthTokenByToken(loginRefreshParam.getToken());
        if (null == oauthTokenOld || sysUser.getId().intValue() != oauthTokenOld.getUserId().intValue()) {
            return ApiResult.fail("token验证失败！");
        }

        SysOrganizeQueryVo sysOrganizeQueryVo = sysOrganizeService.getSysOrganizeById(sysUser.getOrganizeId());
        if (sysOrganizeQueryVo == null) {
            return ApiResult.fail("组织不存在");
        }
        if (!StateEnum.ENABLE.getKey().equals(sysOrganizeQueryVo.getState())) {
            return ApiResult.fail("组织已禁用");
        }

        SysRegion sysRegion = sysRegionService.getById(sysOrganizeQueryVo.getPopedomId());
        if (sysRegion == null) {
            return ApiResult.fail("所属辖区不存在");
        }

        // 获取数据库中保存的盐值
        String newSalt = SaltUtil.getSalt(sysUser.getSalt(), jwtProperties);
        // 生成token字符串并返回
        Long expireSecond = jwtProperties.getExpireSecond();
        String token = JwtUtil.generateToken(sysUser.getId(), sysUser.getUsername(), sysUser.getOrganizeId(), sysUser.getUserType(), sysRegion.getRegionId(), newSalt, Duration.ofSeconds(expireSecond));
        oauthTokenService.removeTokenByUserId(sysUser.getId());
        OauthToken oauthToken = new OauthToken();
        oauthTokenService.removeTokenByUserId(sysUser.getId());
        oauthToken.setAccessToken(token);

        //获取角色id、名称、code
        HashSet<Long> roleIdSet = new HashSet<>();
        HashSet<String> roleNameSet = new HashSet<>();
        HashSet<String> roleCodeSet = new HashSet<>();

        List<SysRoleQueryVo> sysRoleQueryVoList = sysRoleService.getUserRoleByUserId(sysUser.getId());
        if (CollectionUtil.isEmpty(sysRoleQueryVoList)) {
            log.error("角色不存在，用户id:" + sysUser.getId());
        }
        sysRoleQueryVoList.stream().forEach(sysRoleQueryVo -> {
            roleIdSet.add(sysRoleQueryVo.getId());
            roleNameSet.add(sysRoleQueryVo.getName());
            roleCodeSet.add(sysRoleQueryVo.getCode());
        });

        if (!sysRoleQueryVoList.isEmpty()) {
            oauthToken.setRoles(roleCodeSet.toString());
        }
        oauthToken.setRefreshToken(StateEnum.DISABLE.getKey());
        oauthToken.setUserId(sysUser.getId());
        oauthToken.setExpireTime(JwtUtil.getExpireDate(token));
        oauthTokenService.save(oauthToken);
        return ApiResult.ok(token);
    }

    /**
     * 添加用户
     *
     * @param sysUserAddQueryParam
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public ApiResult addSysUser(SysUserAddQueryParam sysUserAddQueryParam) throws Exception {
        // 校验用户名是否存在
        boolean isExists = isExistsByUsername(sysUserAddQueryParam.getUsername());
        if (isExists) {
            return ApiResult.error(StatusCode.SYS_USER_NAME_EXIST);
        }
        SysRole sysRole = sysRoleService.getById(sysUserAddQueryParam.getRoleId());
        //
        List<SysRoleQueryVo> sysRoleQueryVoList = sysRoleService.getUserRoleByUserId(getUserId());
        boolean roleFlag = true;
        for (SysRoleQueryVo sysRoleQueryVo : sysRoleQueryVoList
        ) {
            if (sysRoleQueryVo.getLevel() < sysRole.getLevel()) {
                roleFlag = false;
            }
        }

        if (roleFlag) {
            return ApiResult.fail("非法授权！");
        }

        if (null == sysUserAddQueryParam.getRoleId() || null == sysRole) {
            return ApiResult.error(StatusCode.SYS_USER_ROLE_VERIFY_EXIST);
        }

        SysOrganize sysOrganize = sysOrganizeService.getById((long) 3);
        if (null == sysUserAddQueryParam.getOrganizeId() || null == sysOrganize) {
            return ApiResult.fail("所选机构不存在！");
        }

        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(sysUserAddQueryParam, sysUser);
        sysUser.setUserType(sysOrganize.getOrganizeType());
        String salt = UUIDUtil.getUUID();
        sysUser.setSalt(salt).setCreateUserId(getUserId())
                .setDeleted(StateEnum.DISABLE.getKey()).setCreateUserId(getUserId()).setOrganizeId(sysOrganize.getId());
        // 密码加密
        String newPassword = PasswordUtil.encrypt(sysUser.getPassword(), salt);
        sysUser.setPassword(newPassword);
        sysUser.setState(StateEnum.ENABLE.getKey());
        sysUser.setUpdateTime(new Date());
        this.saveOrUpdate(sysUser);
        //添加角色
        SysUser sysUserNew = this.getSysUserByUserName(sysUser.getUsername());
        SysUserRole sysUserRole = new SysUserRole();
        sysUserRole.setUserId(sysUserNew.getId()).setRoleId(sysRole.getId());
        sysUserRoleService.saveOrUpdate(sysUserRole);
        return ApiResult.ok();
    }


    /**
     * 停用启用用户
     *
     * @param idParam
     * @return
     */
    @Override
    public ApiResult enabledSysUser(IdParam idParam) {
        SysUser sysUser = this.getById(idParam.getId());
        if (null == sysUser) {
            return ApiResult.error(StatusCode.SYS_USER_NAME_EXIST);
        }
        if (sysUser.getState().equals(StateEnum.ENABLE.getKey())) {
            sysUser.setState(StateEnum.DISABLE.getKey());
            oauthTokenService.removeTokenByUserId(sysUser.getId());
        } else {
            sysUser.setState(StateEnum.ENABLE.getKey());
        }
        sysUser.setUpdateTime(new Date());
        this.saveOrUpdate(sysUser);
        return ApiResult.ok();
    }

    /**
     * 删除用户
     *
     * @param idParam
     * @return
     */
    @Override
    @Transactional
    public ApiResult deleteSysUser(IdParam idParam) {
        SysUser sysUser = this.getById(idParam.getId());
        if (null == sysUser) {
            return ApiResult.error(StatusCode.SYS_USER_NAME_EXIST);
        }
        if (StateEnum.ENABLE.getKey().equals(sysUser.getState())) {
            return ApiResult.error(StatusCode.SYS_USER_NAME_STATE_EXIST);
        }
        sysUser.setDeleted(StateEnum.ENABLE.getKey());
        sysUser.setUpdateTime(new Date());
        sysUserRoleService.remove(new QueryWrapper<SysUserRole>().eq("user_id", sysUser.getId()));
        this.saveOrUpdate(sysUser);
        //this.removeById(sysUser.getId());
        return ApiResult.ok();
    }


    @Override
    @Transactional
    public ApiResult updatePassword(UpdatePasswordParam updatePasswordParam) throws Exception {
        if (!verifyUserPermission(updatePasswordParam.getUserId())) {
            return ApiResult.error(StatusCode.AUTH_UPDATE_VERIFY_ERROR);
        }
        String oldPassword = updatePasswordParam.getOldPassword();
        String newPassword = updatePasswordParam.getNewPassword();
        String confirmPassword = updatePasswordParam.getConfirmPassword();
        if (!newPassword.equals(confirmPassword)) {
            return ApiResult.fail("两次输入的密码不一致");
        }
        if (newPassword.equals(oldPassword)) {
            return ApiResult.fail("新密码和旧密码不能一致");
        }

        // 判断原密码是否正确
        SysUser sysUser = getById(updatePasswordParam.getUserId());
        if (sysUser == null) {
            return ApiResult.fail("用户不存在");
        }
        if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
            return ApiResult.fail("用户已禁用");
        }
        // 密码加密处理
        String salt = sysUser.getSalt();
        String encryptOldPassword = PasswordUtil.encrypt(oldPassword, salt);
        if (!sysUser.getPassword().equals(encryptOldPassword)) {
            return ApiResult.fail("原密码错误");
        }
        // 新密码加密
        String encryptNewPassword = PasswordUtil.encrypt(newPassword, salt);
        // 修改密码
        sysUser.setPassword(encryptNewPassword)
                .setUpdateTime(new Date());
        if (updateById(sysUser)) {
            //删除token 登录失效
            oauthTokenService.removeTokenByUserId(sysUser.getId());
            return ApiResult.ok();
        } else {
            return ApiResult.error(StatusCode.SYS_USER_PASSWORD_FAIL);
        }
    }

    /**
     * 修改用户头像
     *
     * @param sysUserUpdateQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public ApiResult updateSysUserHead(SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception {
        if (!verifyUserPermission(sysUserUpdateQueryParam.getId())) {
            return ApiResult.error(StatusCode.AUTH_UPDATE_VERIFY_ERROR);
        }
        SysUser sysUser = new SysUser()
                .setId(sysUserUpdateQueryParam.getId())
                .setHead(sysUserUpdateQueryParam.getHead());
        if (updateById(sysUser)) {
            return ApiResult.ok();
        } else {
            return ApiResult.error(StatusCode.SYS_USER_HEARD_FAIL);
        }
    }

    @Override
    public ApiResult updateUserBack(SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception {
        SysUser sysUser = this.getById(sysUserUpdateQueryParam.getId());
        if (null == sysUser) {
            return ApiResult.error(StatusCode.FORM_SUBMIT_UPDATE_VERIFY);
        }
        String username = sysUser.getUsername();
        BeanUtils.copyProperties(sysUserUpdateQueryParam, sysUser);
        sysUser.setUsername(username);
        if (null != sysUserUpdateQueryParam.getRoleId()) {
            //获取当前用户角色
            List<SysRoleQueryVo> roleList = sysRoleService.getUserRoleByUserId(sysUser.getId());
            boolean roleChange = true;
            for (SysRoleQueryVo sysRoleQueryVo : roleList
            ) {
                if (sysRoleQueryVo.getId() == sysUserUpdateQueryParam.getRoleId()) {
                    roleChange = false;
                }
            }
            if (roleChange) {
                //删除角色
                sysUserRoleService.remove(new QueryWrapper<SysUserRole>().eq("user_id", sysUser.getId()));
                SysRole sysRole = sysRoleService.getById(sysUserUpdateQueryParam.getRoleId());
                if (null == sysUserUpdateQueryParam.getRoleId() || null == sysRole) {
                    return ApiResult.error(StatusCode.SYS_USER_ROLE_VERIFY_EXIST);
                }
                //添加角色
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setUserId(sysUser.getId()).setRoleId(sysRole.getId());
                sysUserRoleService.saveOrUpdate(sysUserRole);
            }
            sysUser.setUpdateTime(new Date());
            this.saveOrUpdate(sysUser);
        }
        return ApiResult.ok();
    }

    @Override
    public ApiResult updateUserNom(SysUserUpdateQueryParam sysUserUpdateQueryParam) throws Exception {
        if (!verifyUserPermission(sysUserUpdateQueryParam.getId())) {
            return ApiResult.error(StatusCode.AUTH_UPDATE_VERIFY_ERROR);
        }

        SysUser sysUser = this.getById(sysUserUpdateQueryParam.getId());
        if (null == sysUser) {
            return ApiResult.error(StatusCode.FORM_SUBMIT_UPDATE_VERIFY);
        }
        String username = sysUser.getUsername();
        BeanUtils.copyProperties(sysUserUpdateQueryParam, sysUser);
        sysUser.setUsername(username);
        this.saveOrUpdate(sysUser);
        return ApiResult.ok();
    }

    /**
     * 重置密码
     *
     * @param updatePasswordParam
     * @return
     * @throws Exception
     */
    @Override
    public ApiResult resetPassword(UpdatePasswordParam updatePasswordParam) throws Exception {
        if (null == updatePasswordParam.getUserId() || StringUtil.isEmpty(updatePasswordParam.getNewPassword())) {
            return ApiResult.fail("请输入重置密码！");
        }
        // 判断原密码是否正确
        SysUser sysUser = getById(updatePasswordParam.getUserId());
        if (sysUser == null) {
            return ApiResult.fail("用户不存在");
        }
        if (StateEnum.DISABLE.getKey().equals(sysUser.getState())) {
            return ApiResult.fail("用户已禁用");
        }
        // 新密码加密
        String encryptNewPassword = PasswordUtil.encrypt(updatePasswordParam.getNewPassword(), sysUser.getSalt());
        // 修改密码
        sysUser.setPassword(encryptNewPassword)
                .setUpdateTime(new Date());
        if (updateById(sysUser)) {
            //删除token 登录失效
            oauthTokenService.removeTokenByUserId(sysUser.getId());
            return ApiResult.ok();
        } else {
            return ApiResult.error(StatusCode.SYS_USER_PASSWORD_FAIL);
        }
    }

    /**
     * 后台获取用户列表
     *
     * @param sysUserQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<SysUserBackQueryVo> getSysUserBackList(SysUserBackQueryParam sysUserQueryParam) throws Exception {
        sysUserQueryParam.setRegionId(getRegionId(sysUserQueryParam.getRegionId()));
        if (OrganizeTypeEnum.ORGANIZE_MANAGE.getCode().equals(getOrganizeType())) {
            sysUserQueryParam.setSystemState(StateEnum.ENABLE.getKey());
        }
        Page page = setPageParam(sysUserQueryParam);
        IPage<SysUserBackQueryVo> iPage = sysUserMapper.getSysUserBackList(page, sysUserQueryParam);
        return new PageTool(iPage);
    }

    /**
     * 用户获取基本信息（用于修改）
     *
     * @return
     */
    @Override
    public ApiResult getSysUserById(HttpServletRequest request) {
        Long userId = getUserId();
        if (null == userId) {
            return ApiResult.fail(ApiCode.TOKEN_INVALID);
        }
        SysUser sysUser = this.getById(userId);
        if (null == sysUser) {
            return ApiResult.error(StatusCode.FORM_SUBMIT_UPDATE_VERIFY);
        }

        SysUserInfoQueryVo sysUserInfoQueryVo = new SysUserInfoQueryVo();
        BeanUtils.copyProperties(sysUser, sysUserInfoQueryVo);

        return ApiResult.ok(sysUserInfoQueryVo);
    }


    @Override
    public PageTool<SysUserQueryVo> getSysUserPageList(SysUserQueryParam sysUserQueryParam) throws Exception {
        Page page = setPageParam(sysUserQueryParam);
        IPage<SysUserQueryVo> iPage = sysUserMapper.getSysUserPageList(page, sysUserQueryParam);
        return new PageTool(iPage);
    }

    @Override
    public boolean isExistsByUsername(String username) throws Exception {
        SysUser selectSysUser = new SysUser().setUsername(username);
        return sysUserMapper.selectCount(new QueryWrapper<>(selectSysUser)) > 0;
    }

    @Override
    public void checkOrganizeAndRole(Long organizeId, Set<Long> roleId) throws Exception {
        // 校验组织是否存在并且可用
        if (StringUtil.longIsNotEmpty(organizeId)) {
            boolean isEnableSysOrganize = sysOrganizeService.isEnableSysOrganize(new IsEnableSysOrganizeParam(null, organizeId));//getUserId(), organizeId));
            if (!isEnableSysOrganize) {
                throw new BusinessException("该组织不存在或已禁用");
            }
        }
        // 校验角色是否存在并且可用
        if (1 == 1) {//CollectionUtil.isNotEmpty(roleId)) {
            boolean isEnable = sysRoleService.isEnableSysRole(roleId);
            if (!isEnable) {
                throw new BusinessException("该角色不存在或已禁用");
            }
        }

    }

    @Override
    public boolean isExistsSysUserByRoleId(Long roleId) throws Exception {
        SysUser sysUser = new SysUser()
                .setState(StateEnum.ENABLE.ordinal());
        return sysUserMapper.selectCount(new QueryWrapper(sysUser)) > 0;
    }


    @Override
    public SysUser getSysUserByUserName(String username) throws Exception {
        return sysUserMapper.getSysUserByUserName(username);
    }

    @Override
    @Transactional
    public ApiResult initUser() {
        /*List<SysOrganize> sysOrganizesList = sysOrganizeService.list(new QueryWrapper<SysOrganize>().eq("organize_type",OrganizeTypeEnum.ORGANIZE_PARK.getCode()));
        for (int i=0;i<sysOrganizesList.size();i++
             ) {
            SysUser sysUser = new SysUser();

            SysOrganize sysOrganize = sysOrganizesList.get(i);

            sysUser.setUserType(OrganizeTypeEnum.ORGANIZE_PARK.getCode());
            sysUser.setNickname(sysOrganize.getOrganizeName());
            sysUser.setState(StateEnum.ENABLE.getKey());
            sysUser.setOrganizeId(sysOrganize.getId());
            sysUser.setCreateUserId(getUserId());
            sysUser.setDeleted(StateEnum.DISABLE.getKey());

            String userName ="park";
            String num= "";
            if(i<10){
                num="00"+i;
            }else if(i>=10&&i<100){
                num = "0"+i;
            }else{
                num = i+"";
            }
            userName = userName+sysOrganize.getPopedomId()+num;
            sysUser.setUsername(userName);
            String salt = UUIDUtil.getUUID();
            sysUser.setSalt(salt)
                    .setDeleted(StateEnum.DISABLE.getKey());
            // 密码加密
            String newPassword = PasswordUtil.encrypt("abc123", salt);
            sysUser.setPassword(newPassword);
            this.saveOrUpdate(sysUser);
            //添加角色
            SysUser sysUserNew = null;
            try {
                sysUserNew = this.getSysUserByUserName(sysUser.getUsername());
            } catch (Exception e) {
                e.printStackTrace();
            }
            SysUserRole sysUserRole = new SysUserRole();
            sysUserRole.setUserId(sysUserNew.getId()).setRoleId((long)3);
            sysUserRoleService.saveOrUpdate(sysUserRole);
        }*/

        List<SysOrganize> sysOrganizesList = sysOrganizeService.list(new QueryWrapper<SysOrganize>().eq("organize_type", OrganizeTypeEnum.ORGANIZE_DISTRICT.getCode()));
        for (int i = 0; i < sysOrganizesList.size(); i++
        ) {
            SysUser sysUser = new SysUser();
            SysOrganize sysOrganize = sysOrganizesList.get(i);
            sysUser.setUserType(OrganizeTypeEnum.ORGANIZE_DISTRICT.getCode());
            sysUser.setNickname(sysOrganize.getOrganizeName());
            sysUser.setState(StateEnum.ENABLE.getKey());
            sysUser.setOrganizeId(sysOrganize.getId());
            sysUser.setCreateUserId(getUserId());
            sysUser.setDeleted(StateEnum.DISABLE.getKey());

            String userName = "user";

            SysRegion sysRegion = sysRegionService.getById(sysOrganize.getPopedomId());

            userName = userName + sysRegion.getRegionId();
            sysUser.setUsername(userName);
            String salt = UUIDUtil.getUUID();
            sysUser.setSalt(salt)
                    .setDeleted(StateEnum.DISABLE.getKey());
            // 密码加密
            String newPassword = PasswordUtil.encrypt("abc123456", salt);
            sysUser.setPassword(newPassword);
            this.saveOrUpdate(sysUser);
            //添加角色
            SysUser sysUserNew = null;
            try {
                sysUserNew = this.getSysUserByUserName(sysUser.getUsername());
            } catch (Exception e) {
                e.printStackTrace();
            }
            SysUserRole sysUserRole = new SysUserRole();
            sysUserRole.setUserId(sysUserNew.getId()).setRoleId((long) 4);
            sysUserRoleService.saveOrUpdate(sysUserRole);
        }
        return ApiResult.ok();
    }

    /**
     * 登录失败日志
     *
     * @return
     */
    public boolean loginFail(String username, String password, String nameValue) {
        //保存日志
        SysAccountLog sysAccountLog = new SysAccountLog();
        //获取请求的类名
        String className = "com.xdf.creative.module.controller.SysUserController";
        //获取请求的方法名
        String methodName = "login";
        sysAccountLog.setMethod(className + "." + methodName);
        //请求的参数
        sysAccountLog.setName("用户登录失败");
        sysAccountLog.setContent("登录账号：" + username + ";密码：" + password + "  原因：" + nameValue);
        sysAccountLog.setCreateTime(new Date());
        //获取用户名
//        if(null==getUserId()) {
//
//        }else{
//            sysAccountLog.setUserId(getUserId());
//        }
        sysAccountLog.setType(LogTypeEnum.LOG_TYPE_FAIL.getCode());
        //获取用户ip地址
        sysAccountLog.setIp(IpUtil.getRequestIp());
        //调用service保存SysLog实体类到数据库
        sysAccountLogService.save(sysAccountLog);
        return true;
    }


    /**
     * 登录成功日志
     *
     * @return
     */
    public boolean loginSuccess(Long userId) {
        //保存日志
        SysAccountLog sysAccountLog = new SysAccountLog();
        //获取请求的类名
        String className = "com.xdf.creative.module.controller.SysUserController";
        //获取请求的方法名
        String methodName = "login";
        sysAccountLog.setMethod(className + "." + methodName);
        //请求的参数
        sysAccountLog.setName("用户登录成功");
        sysAccountLog.setCreateTime(new Date());
        //获取用户名
        sysAccountLog.setUserId(userId);
        sysAccountLog.setType(LogTypeEnum.LOG_TYPE_SUCCESS.getCode());
        //获取用户ip地址
        sysAccountLog.setIp(IpUtil.getRequestIp());
        //调用service保存SysLog实体类到数据库
        sysAccountLogService.save(sysAccountLog);
        return true;
    }
}
