package com.xdf.creative.module.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.WebInitiationApplyQueryParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.WebInitiationApplyExportVo;
import com.xdf.creative.base.vo.creative.WebInitiationApplyQueryVo;
import com.xdf.creative.enums.InitiationApplryAduitStateEnum;
import com.xdf.creative.enums.InitiationApplryAduitTypeEnum;
import com.xdf.creative.enums.StatusCode;
import com.xdf.creative.module.entity.WebInitiationApply;
import com.xdf.creative.module.mapper.WebInitiationApplyMapper;
import com.xdf.creative.module.service.WebInitiationApplyService;
import com.xdf.creative.util.excel.ExportExcel;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 文促会入会申请表 服务实现类
 * </p>
 *
 * @author DeanYe
 * @since 2019-10-30
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class WebInitiationApplyServiceImpl extends BaseServiceImpl<WebInitiationApplyMapper, WebInitiationApply> implements WebInitiationApplyService {

    @Autowired
    private WebInitiationApplyMapper webInitiationApplyMapper;

    @Value("${upload_basePath}")
    private String basePath;

    @Override
    public WebInitiationApplyQueryVo getWebInitiationApplyById(Serializable id) throws Exception {
        return webInitiationApplyMapper.getWebInitiationApplyById(id);
    }

    /**
     * 后台管理获取分页对象
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getWebInitiationApplyPageListBack(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getWebInitiationApplyPageListBack(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    /**
     * web首页获取分页对象
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getWebInitiationApplyPageListHome(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        webInitiationApplyQueryParam.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getWebInitiationApplyPageListHome(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    /**
     * 获取初始审核分页
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getPageListByFirstAudit(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getPageListByFirstAudit(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    ;


    /**
     * 获取信用审核分页
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getPageListByCreditAudit(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        webInitiationApplyQueryParam.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_PASS.getCode());
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getPageListByCreditAudit(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    ;


    /**
     * 获取最终审核分页
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getPageListByFinalAudit(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        webInitiationApplyQueryParam.setCreditAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_PASS.getCode());
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getPageListByFinalAudit(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    ;


    /**
     * 用户提交审核确认
     *
     * @param webInitiationApply
     * @return
     */
    public ApiResult<Boolean> sendAudit(WebInitiationApply webInitiationApply) {
        if (StringUtils.isEmpty(webInitiationApply.getIds())) {
            return ApiResult.error(StatusCode.FORM_SUBMIT_ID_VERIFY);
        }
        if (null == webInitiationApply.getAuditType()) {
            return ApiResult.error(StatusCode.WEB_INITIATION_APPLY_FIRST_AUDIT);
        }
        String[] ids = webInitiationApply.getIds().split(";");
        int successNum = ids.length;
        int failedTotal = 0;
        if (successNum <= 0) {
            return ApiResult.error(StatusCode.WEB_INITIATION_APPLY_SEND_AUDIT);
        }
        boolean successflag = true;
        for (int i = 0; i < successNum; i++) {
            WebInitiationApply webInitiationApplyNew = this.getById(ids[i]);
            if (null == webInitiationApplyNew) {
                failedTotal++;
                successflag = false;
                ;
            } else {
                if (InitiationApplryAduitTypeEnum.INITIATION_APPLRY_ADUIT_TYPE_INIT.getCode().equals(webInitiationApply.getAuditType())) {
                    webInitiationApplyNew.setFirstAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
                    // webInitiationApplyNew.setFirstSubmitTime(new Date());
                    webInitiationApplyNew.setCreditAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_WAIT.getCode());
                } else if (InitiationApplryAduitTypeEnum.INITIATION_APPLRY_ADUIT_TYPE_SECOND.getCode().equals(webInitiationApply.getAuditType())) {
                    webInitiationApplyNew.setCreditAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
                    // webInitiationApplyNew.setCreditSubmitTime(new Date());
                    webInitiationApplyNew.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_WAIT.getCode());
                } else if (InitiationApplryAduitTypeEnum.NITIATION_APPLRY_ADUIT_TYPE_FINAL.getCode().equals(webInitiationApply.getAuditType())) {
                    webInitiationApplyNew.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
                    // webInitiationApplyNew.setFinalSubmitTime(new Date());
                }
                webInitiationApplyNew.setUpdateTime(new Date());
                boolean flag = this.updateById(webInitiationApplyNew);
                if (!flag) {
                    failedTotal++;
                    successflag = false;
                }
            }
        }
        if (successflag) {
            return ApiResult.result(successflag);
        } else {
            return ApiResult.fail(String.format(StatusCode.WEB_INITIATION_APPLY_SEND_SUCCESS.getMsg(), failedTotal));
        }
    }

    /**
     * @description: 导出excel
     * @param: [webInitiationApplyQueryParam]
     * @return: com.xdf.creative.util.page.ApiResult
     * @author  一路心飞扬 lgw
     * @date: 2021/3/1 14:57
     */
    @Override
    public ApiResult getExportPageList(WebInitiationApplyQueryParam webInitiationApplyQueryParam, HttpServletRequest request) {

       List<WebInitiationApplyExportVo> list=baseMapper.getExportPageList(webInitiationApplyQueryParam);
        String[] headers = {"单位名称", "统一社会信用代码 ", "单位类型","省","市","区","详细地址","邮编","网址","法人姓名","法人职务","法人固定电话","法人手机号", "日常联系人姓名", "日常联系人职务", "日常联系电话", "日常联系人手机号", "法人简介", "单位介绍", "专利与荣誉", "金融服务", "产业合作", "企业服务", "其他服务", "行业类别", "申请成为哪级会员","LOGO","执照","申请附件"};
        String fileName = "入会申请列表";
        String excelDirPath = basePath+"/excel/";
        File fileDir = new File(excelDirPath);
        if (!fileDir.isDirectory()) {
            fileDir.mkdirs();
        }
        String excelPath = excelDirPath + fileName + ".xls";// UUIDGenerator.getUUID()
        File file = new File(excelPath);
        try {
            OutputStream out = new FileOutputStream(file);
            ExportExcel<WebInitiationApplyExportVo> excel = new ExportExcel<WebInitiationApplyExportVo>();
            excel.exportExcel(fileName, headers, list, out);
            out.close();
            Map<String, String> mapResult = new HashMap<String, String>();
            mapResult.put("path", "/excel/" + fileName + ".xls");
            mapResult.put("fileName", fileName + ".xls");
            return ApiResult.ok(mapResult);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            return ApiResult.fail("当前记录导出失败，文件创建异常");
        } catch (IOException e) {
            e.printStackTrace();
            return ApiResult.fail("当前记录导出失败，文件读写异常");

        }

    }

}
