package com.xdf.creative.module.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.WebParkEnterApplyEmailParam;
import com.xdf.creative.base.params.dt.WebParkEnterApplyParam;
import com.xdf.creative.base.params.dt.WebParkEnterBackParam;
import com.xdf.creative.base.params.dt.WebParkEnterQueryParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.dt.WebParkEnterQueryVo;
import com.xdf.creative.enums.DeviceEnum;
import com.xdf.creative.enums.OrganizeTypeEnum;
import com.xdf.creative.enums.StatusCode;
import com.xdf.creative.enums.dt.ParkApplyEnum;
import com.xdf.creative.module.entity.WebParkEnter;
import com.xdf.creative.module.mapper.WebParkEnterMapper;
import com.xdf.creative.module.service.WebParkEnterService;
import com.xdf.creative.support.listener.ParkEnterEmailListenEven;
import com.xdf.creative.util.IpUtil;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Date;


/**
 * <p>
 * 园区入会申请 服务实现类
 * </p>
 *
 * @author lgw
 * @since 2020-04-09
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class WebParkEnterServiceImpl extends BaseServiceImpl<WebParkEnterMapper, WebParkEnter> implements WebParkEnterService {

    @Autowired
    private WebParkEnterMapper webParkEnterMapper;


    @Autowired
    private ApplicationContext applicationContext;


    /**
     * 园区入驻申请
     * @param webParkEnterApplyParam
     * @return
     * @throws Exception
     */
    @Override
    public ApiResult apply(WebParkEnterApplyParam webParkEnterApplyParam) throws Exception {
        WebParkEnter webParkEnter = new WebParkEnter();
        BeanUtils.copyProperties(webParkEnterApplyParam,webParkEnter);
        webParkEnter.setCreateTime(new Date()).setProcessState(ParkApplyEnum.PARK_APPLY_AUDIT.getCode());
        this.saveOrUpdate(webParkEnter);

        WebParkEnterApplyEmailParam webParkEnterApplyEmailParam = new WebParkEnterApplyEmailParam();

        BeanUtils.copyProperties(webParkEnter,webParkEnterApplyEmailParam);
        if(IpUtil.getRequestDeviceType()){
            webParkEnterApplyEmailParam.setSource(DeviceEnum.TWO.getKey());
        }else{
            webParkEnterApplyEmailParam.setSource(DeviceEnum.ONE.getKey());
        }
        webParkEnterApplyEmailParam.setParkName(webParkEnter.getParkName());
        applicationContext.publishEvent(new ParkEnterEmailListenEven(webParkEnterApplyEmailParam));

        return ApiResult.ok();
    }

    /**
     * 入驻审核
     * @param webParkEnterBackParam
     * @return
     * @throws Exception
     */
    @Override
    public ApiResult audit(WebParkEnterBackParam webParkEnterBackParam) throws Exception {
        WebParkEnter webParkEnter = this.getById(webParkEnterBackParam.getId());
        if(null==webParkEnter){
            return ApiResult.error(StatusCode.FORM_SUBMIT_ID_VERIFY);
        }

        webParkEnter.setProcessState(webParkEnterBackParam.getProcessState());
        webParkEnter.setAuditUserId(getUserId());
        webParkEnter.setDetail(webParkEnterBackParam.getDetail());
        webParkEnter.setUpdateTime(new Date());
        this.saveOrUpdate(webParkEnter);
        return ApiResult.ok();
    }


    @Override
    public WebParkEnterQueryVo getWebParkEnterById(Serializable id) throws Exception {
        return webParkEnterMapper.getWebParkEnterById(id);
    }



    @Override
    public PageTool<WebParkEnterQueryVo> getWebParkEnterPageList(WebParkEnterQueryParam webParkEnterQueryParam) throws Exception {
        Page page = setPageParam(webParkEnterQueryParam);
        IPage<WebParkEnterQueryVo> iPage = webParkEnterMapper.getWebParkEnterPageList(page, webParkEnterQueryParam);
        return new PageTool(iPage);
    }

}
