package com.xdf.creative.support.aop;

import com.xdf.creative.enums.ApiCode;
import com.xdf.creative.enums.LogTypeEnum;
import com.xdf.creative.module.entity.SysAccountLog;
import com.xdf.creative.module.entity.SysOperateLog;
import com.xdf.creative.module.service.SysAccountLogService;
import com.xdf.creative.module.service.SysOperateLogService;
import com.xdf.creative.util.IpUtil;
import com.xdf.creative.util.ObjectUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.Date;

/**
 * @author : lgw
 * @date : 17:57 2020/3/2
 */
@Aspect
@Component
public class SysAccountLogAspect {
    @Autowired
    private SysAccountLogService sysAccountLogService;
    private static final Logger logger = LoggerFactory.getLogger(SysAccountLogAspect.class);
    //定义切点 @Pointcut
    //在注解的位置切入代码
    @Pointcut("@annotation( com.xdf.creative.support.aop.AccountLogger)")
    public void logAccountPoinCut() {

    }
    //切面 配置通知
    @AfterReturning(value="logAccountPoinCut()",returning="result")
    public void saveAccountLog(JoinPoint joinPoint, Object result) {
        //if(ObjectUtil.getFieldValueByName("code",result).equals(ApiCode.SUCCESS.getCode())){
            //保存日志
            SysAccountLog sysAccountLog = new SysAccountLog();
            //从切面织入点处通过反射机制获取织入点处的方法
            MethodSignature signature = (MethodSignature) joinPoint.getSignature();
            //获取切入点所在的方法
            Method method = signature.getMethod();
            //获取操作
            AccountLogger accountLogger = method.getAnnotation(AccountLogger.class);
            if (accountLogger != null) {
                sysAccountLog.setName(accountLogger.name());//保存获取的操作
            }
            //获取请求的类名
            String className = joinPoint.getTarget().getClass().getName();
            //获取请求的方法名
            String methodName = method.getName();
            sysAccountLog.setMethod(className + "." + methodName);
            //请求的参数
            Object[] args = joinPoint.getArgs();
            //将参数所在的数组转换成json
            String attributeId = ObjectUtil.getFieldValueByName(accountLogger.userName(),args[0]).toString();
            String attributeValue = ObjectUtil.getFieldValueByName(accountLogger.passWord(),args[0]).toString();
            sysAccountLog.setContent("用户登录成功");
            sysAccountLog.setType(LogTypeEnum.LOG_TYPE_SUCCESS.getCode());
            sysAccountLog.setCreateTime(new Date());
            //获取用户名
            sysAccountLog.setUserId(Long.parseLong(attributeId));
            //获取用户ip地址

            sysAccountLog.setIp(IpUtil.getRequestIp());
            //调用service保存SysLog实体类到数据库
            sysAccountLogService.save(sysAccountLog);
     //   }
    }

}
