package com.xdf.creative.support.aop;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.xdf.creative.enums.ApiCode;
import com.xdf.creative.module.entity.SysOperateLog;
import com.xdf.creative.module.service.SysOperateLogService;
import com.xdf.creative.util.IpUtil;
import com.xdf.creative.util.ObjectUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.Date;

/**
 * @author : lgw
 * @date : 17:57 2020/3/2
 */
@Aspect
@Component
public class SysOperateLogAspect {
    @Autowired
    private SysOperateLogService sysOperateLogService;
    private static final Logger logger = LoggerFactory.getLogger(SysOperateLogAspect.class);

    //定义切点 @Pointcut
    //在注解的位置切入代码
    @Pointcut("@annotation( com.xdf.creative.support.aop.OperateLogger)")
    public void logOperatePoinCut() {

    }
    //切面 配置通知
    @AfterReturning(value="logOperatePoinCut()",returning="result")
    public void saveSysLog(JoinPoint joinPoint, Object result) {
        if(ObjectUtil.getFieldValueByName("code",result).equals(ApiCode.SUCCESS.getCode())){
            //保存日志
            SysOperateLog sysOperateLog = new SysOperateLog();
            //从切面织入点处通过反射机制获取织入点处的方法
            MethodSignature signature = (MethodSignature) joinPoint.getSignature();
            //获取切入点所在的方法
            Method method = signature.getMethod();
            //获取操作
            OperateLogger operateLogger = method.getAnnotation(OperateLogger.class);
            if (operateLogger != null) {
                sysOperateLog.setName(operateLogger.name());//保存获取的操作
            }
            //获取请求的类名
            String className = joinPoint.getTarget().getClass().getName();
            //获取请求的方法名
            String methodName = method.getName();
            sysOperateLog.setMethod(className + "." + methodName);
            //请求的参数
            Object[] args = joinPoint.getArgs();
            String params = JSON.toJSONString(args);
            sysOperateLog.setDetail(params);
            //将参数所在的数组转换成json
            StringBuffer contentBuf=new StringBuffer();
            if(null!= ObjectUtil.getFieldValueByName(operateLogger.attributeId(), args[0])) {
                String attributeId = ObjectUtil.getFieldValueByName(operateLogger.attributeId(), args[0]).toString();
                contentBuf.append(operateLogger.attributeId()+":"+attributeId+";");
            }
            if(null!=ObjectUtil.getFieldValueByName(operateLogger.attributeName(), args[0])) {
                String attributeValue = ObjectUtil.getFieldValueByName(operateLogger.attributeName(), args[0]).toString();
                contentBuf.append(operateLogger.attributeName()+":"+attributeValue+";");
            }
            sysOperateLog.setContent(contentBuf.toString());
            sysOperateLog.setCreateTime(new Date());
            //获取用户名
            sysOperateLog.setUserId(sysOperateLogService.getUserId());
            //获取用户ip地址
            sysOperateLog.setIp(IpUtil.getRequestIp());
            //调用service保存SysLog实体类到数据库
            sysOperateLogService.save(sysOperateLog);
        }
    }

}
