package com.xdf.creative.util.tree;

import com.xdf.creative.module.entity.InfoType;
import org.apache.commons.lang.StringUtils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;


/**
 * 系统工具类
 *
 * @author chen
 * @Date 2011-1-26
 */
@SuppressWarnings("unchecked")
public class Utils {

    /**
     * 将list集合转换为TreePanel所使用的数据，并返回
     *
     * @param list   list集合
     * @param config TreeNode的配置信息
     * @return
     */
    public static List tree(List<Object> list, TreeConfig config) {
        List results = new ArrayList();
        if (Utils.isNotEmpty(list) && !config.isNull()) {
            for (int i = 0; i < list.size(); i++) {
                Object obj = list.get(i);
                if (null != obj
                        && getFieldValue(obj, config.getRootField()).equals(
                        config.getRootValue())) {
                    obj = parse(list, obj, config);
                    if (null != obj)
                        results.add(obj);
                }
                if (i == list.size() - 1)
                    return results;
            }
        }
        return results;
    }

    /**
     * 供 @see #tree(List, TreeConfig)方法递归出TreeNode
     *
     * @param list    list集合
     * @param current 当前节点
     * @param config  TreeNode配置信息
     * @return
     */
    @SuppressWarnings("rawtypes")
    public static Object parse(List<Object> list, Object current,
                               TreeConfig config) {
        for (Object obj : list) {

            if (null != obj
                    && getFieldValue(obj, config.getParentField()).equals(
                    getFieldValue(current, config.getIdField()))) {

                if (null != obj) {
                    List children = (List) getFieldValue(current, "children");
                    Object child = parse(list, obj, config);
                    children.add(child);
                }
            }
        }

        if (Utils.isNotEmpty(((List) getFieldValue(current, "children")))) {
            setFieldValue(current, "leaf", false);
        } else {
            setFieldValue(current, "leaf", true);
        }

        return current;
    }


    /**
     * 验证集合是否为空，包括是否为null或者集合长度为0
     *
     * @param obj
     * @return
     * @author chen
     * @Date 2011-4-19
     */
    public static boolean isNotEmpty(Object obj) {
        if (null == obj)
            return false;
        else {
            if (obj instanceof Collection) {
                if (((Collection) obj).size() < 1)
                    return false;
            } else if (obj instanceof Map) {
                if (((Map) obj).size() < 1)
                    return false;
            } else {
                String className = obj.getClass().getName();

                if (className.indexOf(".") != -1)
                    if (((Object[]) obj).length < 1)
                        return false;

                if (className.length() == 2 && className.startsWith("[")) {
                    String type = className.substring(1);
                    if (type.equals("I")) {
                        if (((int[]) obj).length < 1)
                            return false;
                    } else if (type.equals("D")) {
                        if (((double[]) obj).length < 1)
                            return false;
                    } else if (type.equals("F")) {
                        if (((float[]) obj).length < 1)
                            return false;
                    } else if (type.equals("B")) {
                        if (((byte[]) obj).length < 1)
                            return false;
                    } else if (type.equals("C")) {
                        if (((char[]) obj).length < 1)
                            return false;
                    } else if (type.equals("S")) {
                        if (((short[]) obj).length < 1)
                            return false;
                    } else if (type.equals("L")) {
                        if (((long[]) obj).length < 1)
                            return false;
                    } else if (type.equals("Z")) {
                        if (((boolean[]) obj).length < 1)
                            return false;
                    }
                }
            }

            return true;
        }
    }


    /**
     * 得到对象的指定字段值
     *
     * @param obj
     * @param fieldName 字段名
     * @return
     * @author chen
     * @Date 2011-1-26
     */
    public static Object getFieldValue(Object obj, String fieldName) {
        if (null == obj || !StringUtils.isNotEmpty(fieldName))
            return null;
        // 得到对象的所有定义的方法
        Method[] methods = obj.getClass().getDeclaredMethods();

        for (Method method : methods) {
            // 得到对象的getter方法
            if (method.getName().toLowerCase()
                    .equals(("get" + fieldName).toLowerCase()))
                try {
                    // 调用getter方法获得字段值
                    return method.invoke(obj, null);
                } catch (IllegalArgumentException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                    return null;
                } catch (IllegalAccessException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                    return null;
                } catch (InvocationTargetException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                    return null;
                } catch (Exception e) {
                    // TODO: handle exception
                    e.printStackTrace();
                    return null;
                }
        }
        return null;
    }


    /**
     * 设置对象指定的字段值
     *
     * @param obj        需要设置值的对象
     * @param fieldName  字段名
     * @param fieldValue 字段值
     * @author chen
     * @Date 2011-3-10
     */
    public static void setFieldValue(Object obj, String fieldName,
                                     Object fieldValue) {
        if (null == obj || !StringUtils.isNotEmpty(fieldName))
            return;
        // 得到对象的所有定义的方法
        Method[] methods = obj.getClass().getMethods();

        for (Method method : methods) {
            // 得到对象的getter方法
            if (method.getName().toLowerCase()
                    .equals(("set" + fieldName).toLowerCase()))
                try {
                    // 调用setter方法设置字段值
                    method.invoke(obj, fieldValue);
                } catch (IllegalArgumentException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } catch (Exception e) {
                    // TODO: handle exception
                    e.printStackTrace();
                }
        }
    }


}